# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
"""
OpenERP Report Designer
"""
PACKAGE_NAME = 'package'
__name__ = PACKAGE_NAME

import base64
import logging
import os
import random
import re
import socket
import tempfile
import urllib
import xmlrpclib

import pickle

try:
    import cStringIO as StringIO
except:
    import StringIO

import functools

import uno
import pythonloader
import unohelper

from com.sun.star.awt import XActionListener, XItemListener, XTextListener
from com.sun.star.task import XJobExecutor

MODULE_NAME = 'openerp_report'
MODULE_VERSION = '0.1'

COULD_NOT_CONNECT_TO_SERVER = -1
NO_AVAILABLE_DATABASE = 0

DEFAULT_SERVER_URL = "http://localhost:8069"

PROTOCOLS = {
    'XML-RPC': 'http://',
    'XML-RPC secure': 'https://',
    'NET-RPC': 'socket://',
}

logging.basicConfig(
    level=logging.DEBUG,
    filename=os.path.join(tempfile.gettempdir(),
                          'openerp-report-designer.log'),
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)

logger = logging.getLogger('report-designer')

class ConfigClass(dict):
    """
    This class is only for the configuration
    """
    def __getattr__(self, name):
        """
        Allow to the user to fetch a value with instance.value_name
        """
        return self[name]
    def __setattr__(self, name, value):
        """
        Allow to set a value
        """
        self[name] = value
        return self[name]

Configuration = ConfigClass(
    url=DEFAULT_SERVER_URL,
    database='',
    username = '',
    password = '',
    login_status = False,
)

class Connector(object):
    """
    Connector class
    """
    __logger = logging.getLogger('report-designer.connector')

    def __init__(self, hostname, port):
        """
        :param hostname: Host name of the server
        :param port: Port for the connection to the server
        """
        self.hostname = hostname
        self.port = port

class XmlRPC_Connector(Connector):
    """
    This class supports the XmlRPC protocol
    """
    PROTOCOL = 'xmlrpc'
    __logger = logging.getLogger('report-designer.connector.xml-rpc')

    def __init__(self, hostname, port=8069):
        Connector.__init__(self, hostname, port)
        self.url = 'http://%s:%d/xmlrpc' % (self.hostname, self.port)

    def send(self, service_name, method, *args):
        self.__logger.debug('service: %r', service_name)
        self.__logger.debug('method: %r', method)
        self.__logger.debug('args: %r', args)
        url = '%s/%s' % (self.url, service_name)
        service = xmlrpclib.ServerProxy(url)
        return getattr(service, method)(*args)

class NetRPC_Exception(Exception):
    """
    A specific exception for the NetRPC layer
    """
    def __init__(self, faultCode, faultString):
        self.faultCode = faultCode
        self.faultString = faultString
        self.args = (faultCode, faultString)

class NetRPC:
    def __init__(self, sock=None):
        if sock is None:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        else:
            self.sock = sock
        self.sock.settimeout(120)
    def connect(self, host, port=False):
        if not port:
            protocol, buf = host.split('//')
            host, port = buf.split(':')
        self.sock.connect((host, int(port)))

    def disconnect(self):
        self.sock.shutdown(socket.SHUT_RDWR)
        self.sock.close()

    def mysend(self, msg, exception=False, traceback=None):

        msg = pickle.dumps([msg, traceback])
        size = len(msg)
        self.sock.send('%8d' % size)
        self.sock.send(exception and "1" or "0")
        totalsent = 0
        while totalsent < size:
            sent = self.sock.send(msg[totalsent:])
            if sent == 0:
                raise RuntimeError, "socket connection broken"
            totalsent = totalsent + sent
    def myreceive(self):
        buf=''
        while len(buf) < 8:
            chunk = self.sock.recv(8 - len(buf))
            if chunk == '':
                raise RuntimeError, "socket connection broken"
            buf += chunk
        size = int(buf)
        buf = self.sock.recv(1)
        if buf != "0":
            exception = buf
        else:
            exception = False
        msg = ''
        while len(msg) < size:
            chunk = self.sock.recv(size-len(msg))
            if chunk == '':
                raise RuntimeError, "socket connection broken"
            msg = msg + chunk
        msgio = StringIO.StringIO(msg)
        unpickler = pickle.Unpickler(msgio)
        unpickler.find_global = None
        res = unpickler.load()

        if isinstance(res[0], Exception):
            if exception:
                raise NetRPC_Exception(str(res[0]), str(res[1]))
            raise res[0]
        else:
            return res[0]

class NetRPC_Connector(Connector):
    PROTOCOL = 'netrpc'
    __logger = logging.getLogger('report-designer.connector.net-rpc')

    def send(self, service_name, method, *args):
        self.__logger.debug('service: %r', service_name)
        self.__logger.debug('method: %r', method)
        self.__logger.debug('args: %r', args)
        socket = NetRPC()
        socket.connect(self.hostname, self.port)
        socket.mysend((service_name, method, )+args)
        result = socket.myreceive()
        socket.disconnect()
        return result

class Common(object):
    __logger = logging.getLogger('report-designer.service.common')

    def __init__(self, connector=None):
        if connector is None:
            self.connector = Configuration.connector
        else:
            self.connector = connector

    def __getattr__(self, method):
        """
        :param method: The method for the linked object
        """
        self.__logger.debug('method: %r', method)
        def proxy(*args):
            """
            :param args: A list of values for the method
            """
            self.__logger.debug('args: %r', args)
            result = self.connector.send('common', method, *args)
            self.__logger.debug('result: %r' % result)
            return result
        return proxy

class Database(object):
    __logger = logging.getLogger('report-designer.service.database')

    def __init__(self, connector=None):
        if connector is None:
            self.connector = Configuration.connector
        else:
            self.connector = connector

    def __getattr__(self, method):
        """
        :param method: The method for the linked object 
        """
        self.__logger.debug('method: %r', method)
        def proxy(*args):
            """
            :param args: A list of values for the method
            """
            self.__logger.debug('args: %r', args)
            result = self.connector.send('db', method, *args)
            self.__logger.debug('result: %r' % result)
            return result
        return proxy

class Connection(object):
    """
    TODO: Document this class
    """
    __logger = logging.getLogger('report-designer.connection')

    def __init__(self,
                 connector=None,
                 database='openerp',
                 login='admin',
                 password='admin'):
        """
        :param connector:
        :param database:
        :param login:
        :param password:
        """
        if connector is None:
            self.connector = Configuration.connector
        else:
            self.connector = connector

        self.database, self.login, self.password = database, login, password

        self.user_id = Common(self.connector).login(self.database,
                                                    self.login,
                                                    self.password)

    def __repr__(self):
        """
        Return a readable representation of the Connection object
        """
        url = "%(protocol)s://%(login)s:%(password)s@" \
              "%(hostname)s:%(port)d/%(database)s" % {
            'protocol' : self.connector.PROTOCOL,
            'login' : self.login,
            'password' : self.password,
            'hostname' : self.connector.hostname,
            'port' : self.connector.port,
            'database' : self.database,
        }
 
        return "Connection: %s" % url

class Object(object):
    """
    TODO: Document this class
    """
    __logger = logging.getLogger('report-designer.service.object')

    def __repr__(self):
        """
        """
        return "Object <%s>" % (self.model)

    def __init__(self, model, connection=None):
        """
        :param connection:
        :param model:
        """
        if connection is None:
            self.connection = Configuration.connection
        else:
            self.connection = connection
        self.model = model

    def __getattr__(self, method):
        """
        :param method: The method for the linked object
        """
        self.__logger.debug('method: %r', method)
        def proxy(*args):
            """
            :param args: A list of values for the method
            """
            self.__logger.debug(args)
            result = self.connection.connector.send('object', 'execute',
                                                    self.connection.database,
                                                    self.connection.user_id,
                                                    self.connection.password,
                                                    self.model,
                                                    method,
                                                    *args)
            self.__logger.debug('result: %r' % result)
            return result
        return proxy

    def select(self, domain=None, columns=None):
        if domain is None:
            domain = []
        if columns is None:
            columns = []
        return self.read(self.search(domain), columns)

def rpc_factory(url):
    m = re.match('^(http[s]?://|socket://)([\w.\-]+):(\d{1,5})$', url or '')

    host = m.group(2)
    port = m.group(3)
    protocol = m.group(1)
    if not m:
        return -1
    if protocol in ('http://', 'https://'):
        return XmlRPC_Connector(host, int(port))
    elif protocol == 'socket://':
        return NetRPC_Connector(host, int(port))

def login_required(function):
    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        logger.debug("function: %r", function)
        logger.debug("args: %r", args)
        logger.debug("kwargs: %r", kwargs)
        logger.debug("login_status: %r", Configuration.login_status)
        if not Configuration.login_status:
            logger.debug("login_status: %r",
                         Configuration.login_status)
            ServerParameter(None)
            if not Configuration.login_status:
                logger.debug("login_status: %r -> exit",
                             Configuration.login_status)
                exit(1)

        return function(*args, **kwargs)
    return wrapper

#--------------------------------------------------
# An ActionListener adapter.
# This object implements com.sun.star.awt.XActionListener.
# When actionPerformed is called, this will call an arbitrary
#  python procedure, passing it...
#   1. the oActionEvent
#   2. any other parameters you specified to this object's constructor (as a tuple).
class ActionListenerProcAdapter( unohelper.Base, XActionListener ):
    def __init__( self, oProcToCall, tParams=() ):
        self.oProcToCall = oProcToCall # a python procedure
        self.tParams = tParams # a tuple

    # oActionEvent is a com.sun.star.awt.ActionEvent struct.
    def actionPerformed( self, oActionEvent ):
        if callable( self.oProcToCall ):
            apply( self.oProcToCall, (oActionEvent,) + self.tParams )

#--------------------------------------------------
# An ItemListener adapter.
# This object implements com.sun.star.awt.XItemListener.
# When itemStateChanged is called, this will call an arbitrary
#  python procedure, passing it...
#   1. the oItemEvent
#   2. any other parameters you specified to this object's constructor (as a tuple).
class ItemListenerProcAdapter( unohelper.Base, XItemListener ):
    def __init__( self, oProcToCall, tParams=() ):
        self.oProcToCall = oProcToCall # a python procedure
        self.tParams = tParams # a tuple

    # oItemEvent is a com.sun.star.awt.ItemEvent struct.
    def itemStateChanged( self, oItemEvent ):
        if callable( self.oProcToCall ):
            apply( self.oProcToCall, (oItemEvent,) + self.tParams )

#--------------------------------------------------
# An TextListener adapter.
# This object implements com.sun.star.awt.XTextistener.
# When textChanged is called, this will call an arbitrary
#  python procedure, passing it...
#   1. the oTextEvent
#   2. any other parameters you specified to this object's constructor (as a tuple).
class TextListenerProcAdapter( unohelper.Base, XTextListener ):
    def __init__( self, oProcToCall, tParams=() ):
        self.oProcToCall = oProcToCall # a python procedure
        self.tParams = tParams # a tuple

    # oTextEvent is a com.sun.star.awt.TextEvent struct.
    def textChanged( self, oTextEvent ):
        if callable( self.oProcToCall ):
            apply( self.oProcToCall, (oTextEvent,) + self.tParams )

#------------------------------------------------------------
#   Uno ServiceManager access
#   A different version of this routine and global variable
#    is needed for code running inside a component.
#------------------------------------------------------------


# The ServiceManager of the running OOo.
# It is cached in a global variable.
goServiceManager = False
pythonloader.DEBUG = 0
def getServiceManager( cHost="localhost", cPort="2002" ):
    """Get the ServiceManager from the running OpenOffice.org.
        Then retain it in the global variable goServiceManager for future use.
        This is similar to the GetProcessServiceManager() in OOo Basic.
    """
    global goServiceManager
    global pythonloader
    if not goServiceManager:
        # Get the uno component context from the PyUNO runtime
        oLocalContext = uno.getComponentContext()
        # Create the UnoUrlResolver on the Python side.

        # Connect to the running OpenOffice.org and get its context.
        if __name__ != PACKAGE_NAME:
            oLocalResolver = oLocalContext.ServiceManager.createInstanceWithContext(
                "com.sun.star.bridge.UnoUrlResolver", oLocalContext
            )
            oContext = oLocalResolver.resolve(
                "uno:socket,host=%s,port=%d;urp;StarOffice.ComponentContext" % (cHost, cPort)
            )
        # Get the ServiceManager object
            goServiceManager = oContext.ServiceManager
        else:
            goServiceManager = oLocalContext.ServiceManager

    return goServiceManager


#------------------------------------------------------------
#   Uno convenience functions
#   The stuff in this section is just to make
#    python progrmaming of OOo more like using OOo Basic.
#------------------------------------------------------------


# This is the same as ServiceManager.createInstance( ... )
def createUnoService( cClass ):
    """A handy way to create a global objects within the running OOo.
    Similar to the function of the same name in OOo Basic.
    """
    oServiceManager = getServiceManager()
    oObj = oServiceManager.createInstance( cClass )
    return oObj


# The StarDesktop object.  (global like in OOo Basic)
# It is cached in a global variable.
StarDesktop = None
def getDesktop():
    """An easy way to obtain the Desktop object from a running OOo.
    """
    global StarDesktop
    if StarDesktop is None:
        StarDesktop = createUnoService( "com.sun.star.frame.Desktop" )
    return StarDesktop
# preload the StarDesktop variable.
#getDesktop()


# The CoreReflection object.
# It is cached in a global variable.
goCoreReflection = False
def getCoreReflection():
    global goCoreReflection
    if not goCoreReflection:
        goCoreReflection = createUnoService( "com.sun.star.reflection.CoreReflection" )
    return goCoreReflection


def createUnoStruct( cTypeName ):
    """Create a UNO struct and return it.
    Similar to the function of the same name in OOo Basic.
    """
    oCoreReflection = getCoreReflection()
    # Get the IDL class for the type name
    oXIdlClass = oCoreReflection.forName( cTypeName )
    # Create the struct.
    oReturnValue, oStruct = oXIdlClass.createObject( None )
    return oStruct

#------------------------------------------------------------
#   API helpers
#------------------------------------------------------------

def hasUnoInterface( oObject, cInterfaceName ):
    """Similar to Basic's HasUnoInterfaces() function, but singular not plural."""

    # Get the Introspection service.
    oIntrospection = createUnoService( "com.sun.star.beans.Introspection" )

    # Now inspect the object to learn about it.
    oObjInfo = oIntrospection.inspect( oObject )

    # Obtain an array describing all methods of the object.
    oMethods = oObjInfo.getMethods(
        uno.getConstantByName( "com.sun.star.beans.MethodConcept.ALL" )
    )
    # Now look at every method.
    for oMethod in oMethods:
        # Check the method's interface to see if
        #  these aren't the droids you're looking for.
        cMethodInterfaceName = oMethod.getDeclaringClass().getName()
        if cMethodInterfaceName == cInterfaceName:
            return True
    return False

def hasUnoInterfaces( oObject, *cInterfaces ):
    """Similar to the function of the same name in OOo Basic."""
    for cInterface in cInterfaces:
        if not hasUnoInterface( oObject, cInterface ):
            return False
    return True

#------------------------------------------------------------
#   High level general purpose functions
#------------------------------------------------------------


def makePropertyValue( cName=None, uValue=None, nHandle=None, nState=None ):
    """Create a com.sun.star.beans.PropertyValue struct and return it.
    """
    oPropertyValue = createUnoStruct( "com.sun.star.beans.PropertyValue" )

    if cName is not None:
        oPropertyValue.Name = cName
    if uValue is not None:
        oPropertyValue.Value = uValue
    if nHandle is not None:
        oPropertyValue.Handle = nHandle
    if nState is not None:
        oPropertyValue.State = nState

    return oPropertyValue

def Array( *args ):
    """This is just sugar coating so that code from OOoBasic which
    contains the Array() function can work perfectly in python."""
    tArray = ()
    for arg in args:
        tArray += (arg,)
    return tArray


def loadComponentFromURL( cUrl, tProperties=() ):
    """Open or Create a document from it's URL.
    New documents are created from URL's such as:
        private:factory/sdraw
        private:factory/swriter
        private:factory/scalc
        private:factory/simpress
    """
    StarDesktop = getDesktop()
    oDocument = StarDesktop.loadComponentFromURL( cUrl, "_blank", 0, tProperties )
    return oDocument


#------------------------------------------------------------
#   Styles
#------------------------------------------------------------


def defineStyle( oDrawDoc, cStyleFamily, cStyleName, cParentStyleName=None ):
    """Add a new style to the style catalog if it is not already present.
    This returns the style object so that you can alter its properties.
    """

    oStyleFamily = oDrawDoc.getStyleFamilies().getByName( cStyleFamily )

    # Does the style already exist?
    if oStyleFamily.hasByName( cStyleName ):
        # then get it so we can return it.
        oStyle = oStyleFamily.getByName( cStyleName )
    else:
        # Create new style object.
        oStyle = oDrawDoc.createInstance( "com.sun.star.style.Style" )

        # Set its parent style
        if cParentStyleName is not None:
            oStyle.setParentStyle( cParentStyleName )

        # Add the new style to the style family.
        oStyleFamily.insertByName( cStyleName, oStyle )

    return oStyle


def getStyle( oDrawDoc, cStyleFamily, cStyleName ):
    """Lookup and return a style from the document.
    """
    return oDrawDoc.getStyleFamilies().getByName( cStyleFamily ).getByName( cStyleName )

#------------------------------------------------------------
#   General Utility functions
#------------------------------------------------------------


def convertToURL( cPathname ):
    """Convert a Windows or Linux pathname into an OOo URL."""
    if len( cPathname ) > 1:
        if cPathname[1:2] == ":":
            cPathname = "/" + cPathname[0] + "|" + cPathname[2:]
    cPathname = cPathname.replace( "\\", "/" )
    cPathname = "file://" + cPathname
    return cPathname

# The global Awt Toolkit.
# This is initialized the first time it is needed.
#goAwtToolkit = createUnoService( "com.sun.star.awt.Toolkit" )
goAwtToolkit = None

def getAwtToolkit():
    global goAwtToolkit
    if goAwtToolkit is None:
        goAwtToolkit = createUnoService( "com.sun.star.awt.Toolkit" )
    return goAwtToolkit

# This class builds dialog boxes.
# This can be used in two different ways...
# 1. by subclassing it (elegant)
# 2. without subclassing it (less elegant)
class DBModalDialog(object):
    """Class to build a dialog box from the com.sun.star.awt.* services.
    This doesn't do anything you couldn't already do using OOo's UNO API,
     this just makes it much easier.
    You can change the dialog box size, position, title, etc.
    You can add controls, and listeners for those controls to the dialog box.
    This class can be used by subclassing it, or without subclassing it.
    """
    def __init__(self,
                 nPositionX=None,
                 nPositionY=None,
                 nWidth=None,
                 nHeight=None,
                 cTitle=None):
        self.oDialogModel = createUnoService( "com.sun.star.awt.UnoControlDialogModel" )
        if nPositionX is not None:  self.oDialogModel.PositionX = nPositionX
        if nPositionY is not None:  self.oDialogModel.PositionY = nPositionY
        if nWidth     is not None:  self.oDialogModel.Width     = nWidth
        if nHeight    is not None:  self.oDialogModel.Height    = nHeight
        if cTitle     is not None:  self.oDialogModel.Title     = cTitle
        self.oDialogControl = createUnoService( "com.sun.star.awt.UnoControlDialog" )
        self.oDialogControl.setModel( self.oDialogModel )

    def release( self ):
        """Release resources.
        After calling this, you can no longer use this object.
        """
        self.oDialogControl.dispose()

    #--------------------------------------------------
    #   Dialog box adjustments
    #--------------------------------------------------

    def setDialogPosition( self, nX, nY ):
        self.oDialogModel.PositionX = nX
        self.oDialogModel.PositionY = nY

    def setDialogSize( self, nWidth, nHeight ):
        self.oDialogModel.Width = nWidth
        self.oDialogModel.Height = nHeight

    def setDialogTitle( self, cCaption ):
        self.oDialogModel.Title = cCaption

    def setVisible( self, bVisible ):
        self.oDialogControl.setVisible( bVisible )


    #--------------------------------------------------
    #   com.sun.star.awt.UnoControlButton
    #--------------------------------------------------

    # After you add a Button control, you can call self.setControlModelProperty()
    #  passing any of the properties for a...
    #       com.sun.star.awt.UnoControlButtonModel
    #       com.sun.star.awt.UnoControlDialogElement
    #       com.sun.star.awt.UnoControlModel
    def addButton( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                       cLabel=None,
                       actionListenerProc=None,
                       nTabIndex=None ):
        self.addControl("com.sun.star.awt.UnoControlButtonModel",
                        cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        bDropdown=None,
                        bMultiSelection=None,
                        cLabel=cLabel,
                        nTabIndex=nTabIndex)
        if actionListenerProc is not None:
            self.addActionListenerProc( cCtrlName, actionListenerProc )

    def setButtonLabel( self, cCtrlName, cLabel ):
        """Set the label of the control."""
        oControl = self.getControl( cCtrlName )
        oControl.setLabel( cLabel )

    #--------------------------------------------------
    #   com.sun.star.awt.UnoControlEditModel
    #--------------------------------------------------
    def addEdit( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        cText=None,
                        textListenerProc=None ):
        """Add a Edit control to the window."""
        self.addControl( "com.sun.star.awt.UnoControlEditModel",
            cCtrlName, nPositionX, nPositionY, nWidth, nHeight, bDropdown=None)

        if cText is not None:
            self.setEditText( cCtrlName, cText )
        if textListenerProc is not None:
            self.addTextListenerProc( cCtrlName, textListenerProc )

    #--------------------------------------------------
    #   com.sun.star.awt.UnoControlCheckBox
    #--------------------------------------------------

    # After you add a CheckBox control, you can call self.setControlModelProperty()
    #  passing any of the properties for a...
    #       com.sun.star.awt.UnoControlCheckBoxModel
    #       com.sun.star.awt.UnoControlDialogElement
    #       com.sun.star.awt.UnoControlModel
    def addCheckBox( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                       cLabel=None,
                       itemListenerProc=None,
                       nTabIndex=None ):
        self.addControl("com.sun.star.awt.UnoControlCheckBoxModel",
                        cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        bDropdown=None,
                        bMultiSelection=None,
                        cLabel=cLabel,
                        nTabIndex=nTabIndex )
        if itemListenerProc is not None:
            self.addItemListenerProc( cCtrlName, itemListenerProc )

    def setEditText( self, cCtrlName, cText ):
        """Set the text of the edit box."""
        oControl = self.getControl( cCtrlName )
        oControl.setText( cText )

    def getEditText( self, cCtrlName):
        """Set the text of the edit box."""
        oControl = self.getControl( cCtrlName )
        return oControl.getText()

    def setCheckBoxLabel( self, cCtrlName, cLabel ):
        """Set the label of the control."""
        oControl = self.getControl( cCtrlName )
        oControl.setLabel( cLabel )

    def getCheckBoxState( self, cCtrlName ):
        """Get the state of the control."""
        oControl = self.getControl( cCtrlName )
        return oControl.getState()

    def setCheckBoxState( self, cCtrlName, nState ):
        """Set the state of the control."""
        oControl = self.getControl( cCtrlName )
        oControl.setState( nState )

    def enableCheckBoxTriState( self, cCtrlName, bTriStateEnable ):
        """Enable or disable the tri state mode of the control."""
        oControl = self.getControl( cCtrlName )
        oControl.enableTriState( bTriStateEnable )


    #--------------------------------------------------
    #   com.sun.star.awt.UnoControlFixedText
    #--------------------------------------------------

    def addFixedText( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        cLabel=None ):
        self.addControl( "com.sun.star.awt.UnoControlFixedTextModel",
                         cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                         bDropdown=None, bMultiSelection=None,
                         cLabel=cLabel )

        return self.getControl( cCtrlName )

    #--------------------------------------------------
    #   Add Controls to dialog
    #--------------------------------------------------

    def addControl( self, cCtrlServiceName,
                        cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        bDropdown=None,
                        bMultiSelection=None,
                        cLabel=None,
                        nTabIndex=None,
                        sImagePath=None,
                         ):
        oControlModel = self.oDialogModel.createInstance( cCtrlServiceName )
        self.oDialogModel.insertByName( cCtrlName, oControlModel )
        # if negative coordinates are given for X or Y position,
        #  then make that coordinate be relative to the right/bottom
        #  edge of the dialog box instead of to the left/top.
        if nPositionX < 0:
            nPositionX = self.oDialogModel.Width  + nPositionX - nWidth
        if nPositionY < 0:
            nPositionY = self.oDialogModel.Height + nPositionY - nHeight
        oControlModel.PositionX = nPositionX
        oControlModel.PositionY = nPositionY
        oControlModel.Width = nWidth
        oControlModel.Height = nHeight
        oControlModel.Name = cCtrlName

        if bDropdown is not None:
            oControlModel.Dropdown = bDropdown

        if bMultiSelection is not None:
            oControlModel.MultiSelection = bMultiSelection

        if cLabel is not None:
            oControlModel.Label = cLabel

        if nTabIndex is not None:
            oControlModel.TabIndex = nTabIndex

        if sImagePath is not None:
            oControlModel.ImageURL = sImagePath
    #--------------------------------------------------
    #   Access controls and control models
    #--------------------------------------------------

    #--------------------------------------------------
    #   com.sun.star.awt.UnoContorlListBoxModel
    #--------------------------------------------------


    def addComboListBox( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        bDropdown=True,
                        bMultiSelection=False,
                        itemListenerProc=None,
                        actionListenerProc=None,
                        ):

        self.addControl("com.sun.star.awt.UnoControlListBoxModel",
                        cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        bDropdown, bMultiSelection )

        if itemListenerProc is not None:
            self.addItemListenerProc( cCtrlName, itemListenerProc )

    def addListBoxItems( self, cCtrlName, tcItemTexts, nPosition=0 ):
        """Add a tupple of items to the ListBox at specified position."""
        oControl = self.getControl( cCtrlName )
        oControl.addItems( tcItemTexts, nPosition )

    def selectListBoxItem( self, cCtrlName, cItemText, bSelect=True ):
        """Selects/Deselects the ispecified item."""
        oControl = self.getControl( cCtrlName )
        return oControl.selectItem( cItemText, bSelect )

    def selectListBoxItemPos( self, cCtrlName, nItemPos, bSelect=True ):
        """Select/Deselect the item at the specified position."""
        oControl = self.getControl( cCtrlName )
        return oControl.selectItemPos( nItemPos, bSelect )

    def removeListBoxItems( self, cCtrlName, nPosition, nCount=1 ):
        """Remove items from a ListBox."""
        oControl = self.getControl( cCtrlName )
        oControl.removeItems( nPosition, nCount )

    def getListBoxItemCount( self, cCtrlName ):
        """Get the number of items in a ListBox."""
        oControl = self.getControl( cCtrlName )
        return oControl.getItemCount()

    def getListBoxSelectedItem( self, cCtrlName ):
        """Returns the currently selected item."""
        oControl = self.getControl( cCtrlName )
        return oControl.getSelectedItem()

    def getListBoxItem( self, cCtrlName, nPosition ):
        """Return the item at specified position within the ListBox."""
        oControl = self.getControl( cCtrlName )
        return oControl.getItem( nPosition )

    def getListBoxSelectedItemPos(self, cCtrlName):

        oControl = self.getControl( cCtrlName )
        return oControl.getSelectedItemPos()

    def getListBoxSelectedItems(self, cCtrlName):
        oControl = self.getControl( cCtrlName )
        return oControl.getSelectedItems()

    def getListBoxSelectedItemsPos(self, cCtrlName):

        oControl = self.getControl( cCtrlName )
        return oControl.getSelectedItemsPos()

    #--------------------------------------------------
    #   com.sun.star.awt.UnoControlComboBoxModel
    #--------------------------------------------------
    def addComboBox( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        bDropdown=True,
                        itemListenerProc=None,
                        actionListenerProc=None ):

        self.addControl("com.sun.star.awt.UnoControlComboBoxModel",
                        cCtrlName, nPositionX, nPositionY, nWidth, nHeight, bDropdown)
        if itemListenerProc is not None:
            self.addItemListenerProc( cCtrlName, itemListenerProc )
        if actionListenerProc is not None:
            self.addActionListenerProc( cCtrlName, actionListenerProc )


    def setComboBoxText( self, cCtrlName, cText ):
        """Set the text of the ComboBox."""
        oControl = self.getControl( cCtrlName )
        oControl.setText( cText )

    def getComboBoxText( self, cCtrlName):
        """Set the text of the ComboBox."""
        oControl = self.getControl( cCtrlName )
        return oControl.getText()

    def getComboBoxSelectedText( self, cCtrlName ):
        """Get the selected text of the ComboBox."""
        oControl = self.getControl( cCtrlName )
        return oControl.getSelectedText()

    def getControl( self, cCtrlName ):
        """Get the control (not its model) for a particular control name.
        The control returned includes the service com.sun.star.awt.UnoControl,
         and another control-specific service which inherits from it.
        """
        oControl = self.oDialogControl.getControl( cCtrlName )
        return oControl

    def getControlModel( self, cCtrlName ):
        """Get the control model (not the control) for a particular control name.
        The model returned includes the service UnoControlModel,
         and another control-specific service which inherits from it.
        """
        oControl = self.getControl( cCtrlName )
        oControlModel = oControl.getModel()
        return oControlModel
    #---------------------------------------------------
    #    com.sun.star.awt.UnoControlImageControlModel
    #---------------------------------------------------
    def addImageControl( self, cCtrlName, nPositionX, nPositionY, nWidth, nHeight,
                        sImagePath="",
                        itemListenerProc=None,
                        actionListenerProc=None ):

        self.addControl("com.sun.star.awt.UnoControlImageControlModel",
                        cCtrlName, nPositionX, nPositionY, nWidth, nHeight, sImagePath=sImagePath)

        if itemListenerProc is not None:
            self.addItemListenerProc( cCtrlName, itemListenerProc )
        if actionListenerProc is not None:
            self.addActionListenerProc( cCtrlName, actionListenerProc )


    #--------------------------------------------------
    #   Adjust properties of control models
    #--------------------------------------------------

    def setControlModelProperty( self, cCtrlName, cPropertyName, uValue ):
        """Set the value of a property of a control's model.
        This affects the control model, not the control.
        """
        oControlModel = self.getControlModel( cCtrlName )
        oControlModel.setPropertyValue( cPropertyName, uValue )

    def getControlModelProperty( self, cCtrlName, cPropertyName ):
        """Get the value of a property of a control's model.
        This affects the control model, not the control.
        """
        oControlModel = self.getControlModel( cCtrlName )
        return oControlModel.getPropertyValue( cPropertyName )

    #--------------------------------------------------
    #   Sugar coated property adjustments to control models.
    #--------------------------------------------------

    def setEnabled( self, cCtrlName, bEnabled=True ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        self.setControlModelProperty( cCtrlName, "Enabled", bEnabled )

    def getEnabled( self, cCtrlName ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """

        return self.getControlModelProperty( cCtrlName, "Enabled" )

    def setState( self, cCtrlName, nState ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        self.setControlModelProperty( cCtrlName, "State", nState )

    def getState( self, cCtrlName ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        return self.getControlModelProperty( cCtrlName, "State" )

    def setLabel( self, cCtrlName, cLabel ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        self.setControlModelProperty( cCtrlName, "Label", cLabel )

    def getLabel( self, cCtrlName ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        return self.getControlModelProperty( cCtrlName, "Label" )

    def setHelpText( self, cCtrlName, cHelpText ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        self.setControlModelProperty( cCtrlName, "HelpText", cHelpText )

    def getHelpText( self, cCtrlName ):
        """Supported controls...
            UnoControlButtonModel
            UnoControlCheckBoxModel
        """
        return self.getControlModelProperty( cCtrlName, "HelpText" )


    #--------------------------------------------------
    #   Adjust controls (not models)
    #--------------------------------------------------

    # The following apply to all controls which are a
    #   com.sun.star.awt.UnoControl

    def setDesignMode( self, cCtrlName, bDesignMode=True ):
        oControl = self.getControl( cCtrlName )
        oControl.setDesignMode( bDesignMode )

    def isDesignMode( self, cCtrlName, bDesignMode=True ):
        oControl = self.getControl( cCtrlName )
        return oControl.isDesignMode()

    def isTransparent( self, cCtrlName, bDesignMode=True ):
        oControl = self.getControl( cCtrlName )
        return oControl.isTransparent()


    # The following apply to all controls which are a
    #   com.sun.star.awt.UnoControlDialogElement

    def setPosition( self, cCtrlName, nPositionX, nPositionY ):
        self.setControlModelProperty( cCtrlName, "PositionX", nPositionX )
        self.setControlModelProperty( cCtrlName, "PositionY", nPositionY )
    def setPositionX( self, cCtrlName, nPositionX ):
        self.setControlModelProperty( cCtrlName, "PositionX", nPositionX )
    def setPositionY( self, cCtrlName, nPositionY ):
        self.setControlModelProperty( cCtrlName, "PositionY", nPositionY )
    def getPositionX( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "PositionX" )
    def getPositionY( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "PositionY" )

    def setSize( self, cCtrlName, nWidth, nHeight ):
        self.setControlModelProperty( cCtrlName, "Width", nWidth )
        self.setControlModelProperty( cCtrlName, "Height", nHeight )
    def setWidth( self, cCtrlName, nWidth ):
        self.setControlModelProperty( cCtrlName, "Width", nWidth )
    def setHeight( self, cCtrlName, nHeight ):
        self.setControlModelProperty( cCtrlName, "Height", nHeight )
    def getWidth( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "Width" )
    def getHeight( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "Height" )

    def setTabIndex( self, cCtrlName, nWidth, nTabIndex ):
        self.setControlModelProperty( cCtrlName, "TabIndex", nTabIndex )
    def getTabIndex( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "TabIndex" )

    def setStep( self, cCtrlName, nWidth, nStep ):
        self.setControlModelProperty( cCtrlName, "Step", nStep )
    def getStep( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "Step" )

    def setTag( self, cCtrlName, nWidth, cTag ):
        self.setControlModelProperty( cCtrlName, "Tag", cTag )
    def getTag( self, cCtrlName ):
        return self.getControlModelProperty( cCtrlName, "Tag" )

    def setEchoChar(self, cCtrlName , cVal):
        self.setControlModelProperty(cCtrlName, "EchoChar", cVal)
    def getEchoChar(self, cCtrlName):
        return self.setControlModelProperty(cCtrlName, "EchoChar")

    #--------------------------------------------------
    #   Add listeners to controls.
    #--------------------------------------------------

    # This applies to...
    #   UnoControlButton
    def addActionListenerProc( self, cCtrlName, actionListenerProc ):
        """Create an com.sun.star.awt.XActionListener object and add it to a control.
        A listener object is created which will call the python procedure actionListenerProc.
        The actionListenerProc can be either a method or a global procedure.
        The following controls support XActionListener:
            UnoControlButton
        """
        oControl = self.getControl( cCtrlName )
        oActionListener = ActionListenerProcAdapter( actionListenerProc )
        oControl.addActionListener( oActionListener )

    # This applies to...
    #   UnoControlCheckBox
    def addItemListenerProc( self, cCtrlName, itemListenerProc ):
        """Create an com.sun.star.awt.XItemListener object and add it to a control.
        A listener object is created which will call the python procedure itemListenerProc.
        The itemListenerProc can be either a method or a global procedure.
        The following controls support XActionListener:
            UnoControlCheckBox
        """
        oControl = self.getControl( cCtrlName )
        oActionListener = ItemListenerProcAdapter( itemListenerProc )
        oControl.addItemListener( oActionListener )

    #--------------------------------------------------
    #   Display the modal dialog.
    #--------------------------------------------------

    def doModalDialog( self, sObjName, sValue):
        """Display the dialog as a modal dialog."""
        self.oDialogControl.setVisible( True )
        if not sValue is None:
            self.selectListBoxItem( sObjName, sValue, True )
        self.oDialogControl.execute()

    def endExecute( self ):
        """Call this from within one of the listeners to end the modal dialog.
        For instance, the listener on your OK or Cancel button would call this to end the dialog.
        """
        self.oDialogControl.endExecute()


class ErrorDialog(DBModalDialog):
    def __init__(self, sErrorMsg, sErrorHelpMsg="", sTitle="Error Message"):
        DBModalDialog.__init__(self, 50, 50, 150, 90, sTitle)
        self.addFixedText("lblErrMsg", 5, 5, 190, 25, sErrorMsg)
        self.addFixedText("lblErrHelpMsg", 5, 30, 190, 25, sErrorHelpMsg)
        self.addButton('btnOK', 55, -5, 40, 15, 'Ok',
                       actionListenerProc = self.btnOkOrCancel_clicked )
        self.doModalDialog("", None)

    def btnOkOrCancel_clicked( self, oActionEvent ):
        self.endExecute()

def genTree(object_name, aList, insField, host, level=3, ending=[],
            ending_excl=[],
            recur=[],
            root='',
            actualroot=""):
    try:
        res = Object(object_name).fields_get()
        for key in sorted(res.keys()):
            if (not ending or res[key]['type'] in ending) \
               and ((not ending_excl) or not (res[key]['type'] in ending_excl)):
                insField.addItem(root+'/'+res[key]["string"], len(aList))
                aList.append(actualroot+'/'+key)
            if (res[key]['type'] in recur) and (level>0):
                genTree(res[key]['relation'], aList, insField, host, level-1,
                        ending, ending_excl, recur,
                        root+'/'+res[key]["string"],
                        actualroot+'/'+key)

    except Exception, exception:
        logger.exception(exception)

def VariableScope(oTcur, insVariable, aObjectList, aComponentAdd, aItemList, sTableName=""):
    if sTableName.find(".") != -1:
        for i in range(len(aItemList)):
            if aComponentAdd[i] == sTableName:
                sLVal = aItemList[i][1][aItemList[i][1].find(",'")+2:aItemList[i][1].find("')")]
                for j in range(len(aObjectList)):
                    if aObjectList[j][:aObjectList[j].find("(")] == sLVal:
                        insVariable.append(aObjectList[j])
        VariableScope(oTcur, insVariable, aObjectList, aComponentAdd,
                      aItemList, sTableName[:sTableName.rfind(".")]
                     )
    else:
        for i in range(len(aItemList)):

            if aComponentAdd[i] == sTableName:
                sLVal = aItemList[i][1][aItemList[i][1].find(",'")+2:aItemList[i][1].find("')")]

                for j in range(len(aObjectList)):
                    if aObjectList[j][:aObjectList[j].find("(")] == sLVal and sLVal != "":
                        insVariable.append(aObjectList[j])

def getList(aObjectList, host, count):
    desktop = getDesktop()
    doc = desktop.getCurrentComponent()
    docinfo = doc.getDocumentInfo()
    sMain=""
    if count >= 1:
        oParEnum = doc.getTextFields().createEnumeration()
        while oParEnum.hasMoreElements():
            oPar = oParEnum.nextElement()
            if not oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                continue
            sItem = oPar.Items[1]
            if not sItem[sItem.find("(")+1:sItem.find(",")] == "objects":
                continue

            sMain = sItem[sItem.find(",'")+2:sItem.find("')")]
            oParEnum = doc.getTextFields().createEnumeration()
            while oParEnum.hasMoreElements():
                oPar = oParEnum.nextElement()
                if not oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                    continue

                sItem = oPar.Items[1]
                if not sItem[sItem.find("[[ ")+3:sItem.find("(")] == "repeatIn":
                    continue

                if sItem[sItem.find("(")+1:sItem.find(",")] == "objects":
                    aObjectList.append(sItem[sItem.rfind(",'")+2:sItem.rfind("')")] + "(" + docinfo.getUserFieldValue(3) + ")")
                else:
                    sTemp = sItem[sItem.find("(")+1:sItem.find(",")]
                    if sMain == sTemp[:sTemp.find(".")]:
                        getRelation(docinfo.getUserFieldValue(3), sItem[sItem.find(".")+1:sItem.find(",")], sItem[sItem.find(",'")+2:sItem.find("')")], aObjectList, host)
                    else:
                        sPath = getPath(sItem[sItem.find("(")+1:sItem.find(",")], sMain)
                        getRelation(docinfo.getUserFieldValue(3), sPath, sItem[sItem.find(",'")+2:sItem.find("')")], aObjectList, host)
    else:
        aObjectList.append("List of " + docinfo.getUserFieldValue(3))

def getRelation(sRelName, sItem, sObjName, aObjectList, host ):
    res = Object(sRelName).fields_get()
    keys = res.keys()
    for key in keys:
        if sItem.find(".") == -1:
            if key == sItem:
                aObjectList.append(sObjName + "(" + res[key]['relation'] + ")")
                return 0
        if key == sItem[:sItem.find(".")]:
            getRelation(res[key]['relation'], sItem[sItem.find(".")+1:], sObjName, aObjectList, host)


def getPath(sPath, sMain):
    desktop = getDesktop()
    doc = desktop.getCurrentComponent()
    oParEnum = doc.getTextFields().createEnumeration()
    while oParEnum.hasMoreElements():
        oPar = oParEnum.nextElement()
        if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
            sItem = oPar.Items[1]
            if sPath[:sPath.find(".")] == sMain:
                break
            else:
                res = re.findall('\\[\\[ *([a-zA-Z0-9_\.]+) *\\]\\]', sPath)
                if len(res):
                    if sItem[sItem.find(",'")+2:sItem.find("')")] == sPath[:sPath.find(".")]:
                        sPath =  sItem[sItem.find("(")+1:sItem.find(",")] + sPath[sPath.find("."):]
                        getPath(sPath, sMain)
    return sPath

def EnumDocument(aItemList, aComponentAdd):
    desktop = getDesktop()
    parent=""
    Doc = desktop.getCurrentComponent()
    #oVC = Doc.CurrentController.getViewCursor()
    oParEnum = Doc.getTextFields().createEnumeration()
    while oParEnum.hasMoreElements():
        oPar = oParEnum.nextElement()
        if oPar.Anchor.TextTable:
            #parent = oPar.Anchor.TextTable.Name
            getChildTable(oPar.Anchor.TextTable, aItemList, aComponentAdd)
        elif oPar.Anchor.TextSection:
            parent = oPar.Anchor.TextSection.Name
        elif oPar.Anchor.Text:
            parent = "Document"
        sItem = oPar.Items[1].replace(' ',"")
        if sItem[sItem.find("[[ ")+3:sItem.find("(")] == "repeatIn" and not oPar.Items in aItemList:
            templist = oPar.Items[0], sItem
            aItemList.append( templist )
        aComponentAdd.append( parent )

def getChildTable(oPar, aItemList, aComponentAdd, sTableName=""):
    sNames = oPar.getCellNames()
    bEmptyTableFlag = True
    for val in sNames:
        oCell = oPar.getCellByName(val)
        oCurEnum = oCell.createEnumeration()
        while oCurEnum.hasMoreElements():
            try:
                oCur = oCurEnum.nextElement()

                if oCur.supportsService("com.sun.star.text.TextTable"):
                    if sTableName == "":
                        getChildTable(oCur, aItemList, aComponentAdd, oPar.Name)
                    else:
                        getChildTable(oCur, aItemList, aComponentAdd, sTableName+"."+oPar.Name)
                else:
                    oSecEnum = oCur.createEnumeration()
                    while oSecEnum.hasMoreElements():
                        oSubSection = oSecEnum.nextElement()
                        if not oSubSection.supportsService("com.sun.star.text.TextField"):
                            continue
                        bEmptyTableFlag = False
                        sItem = oSubSection.TextField.Items[1]
                        if not sItem[sItem.find("[[ ")+3:sItem.find("(")] == "repeatIn":
                            continue

                        if aItemList.__contains__(oSubSection.TextField.Items) == False:
                            aItemList.append(oSubSection.TextField.Items)
                        if sTableName == "":
                            if  aComponentAdd.__contains__(oPar.Name) == False:
                                aComponentAdd.append(oPar.Name)
                        else:
                            if aComponentAdd.__contains__(sTableName+"."+oPar.Name) == False:
                                aComponentAdd.append(sTableName+"."+oPar.Name)
            except Exception, exception:
                logger.exception(exception)

    if bEmptyTableFlag == True:
        aItemList.append((u'', u''))
        if sTableName == "":
            if not oPar.Name in aComponentAdd:
                aComponentAdd.append(oPar.Name)
        else:
            if not (sTableName+"."+oPar.Name) in aComponentAdd:
                aComponentAdd.append(sTableName+"."+oPar.Name)
    return 0

def getRecursiveSection(oCurrentSection, aSectionList):
    desktop = getDesktop()
    doc = desktop.getCurrentComponent()
    doc.getText().createEnumeration()
    aSectionList.append(oCurrentSection.Name)
    if oCurrentSection.ParentSection:
        getRecursiveSection(oCurrentSection.ParentSection, aSectionList)

def get_absolute_file_path( url ):
    url_unquoted = urllib.unquote(url)
    return os.name == 'nt' and url_unquoted[1:] or url_unquoted 

# This function reads the content of a file and return it to the caller
def read_data_from_file( filename ):
    fp = file( filename, "rb" )
    data = fp.read()
    fp.close()
    return data

# This function writes the content to a file
def write_data_to_file( filename, data ):
    fp = file( filename, 'wb' )
    fp.write( data )
    fp.close()


class RepeatIn( unohelper.Base, XJobExecutor, DBModalDialog):
    @login_required
    def __init__(self, sObject="", sVariable="", sFields="", sDisplayName="",
                 bFromModify=False):
        DBModalDialog.__init__(self, 60, 50, 180, 250, "RepeatIn Builder")
        # Interface Design
        self.addFixedText("lblVariable", 2, 12, 60, 15, "Objects to loop on :")
        self.addComboBox("cmbVariable", 180-120-2, 10, 120, 15, True,
                         itemListenerProc=self.cmbVariable_selected)
        self.insVariable = self.getControl( "cmbVariable" )

        self.addFixedText("lblFields", 10, 32, 60, 15, "Field to loop on :")
        self.addComboListBox("lstFields", 180-120-2, 30, 120, 150, False,
                             itemListenerProc=self.lstbox_selected)
        self.insField = self.getControl( "lstFields" )

        self.addFixedText("lblName", 12, 187, 60, 15, "Variable name :")
        self.addEdit("txtName", 180-120-2, 185, 120, 15,)

        self.addFixedText("lblUName", 8, 207, 60, 15, "Displayed name :")
        self.addEdit("txtUName", 180-120-2, 205, 120, 15,)

        self.addButton('btnOK', -2 , -10, 45, 15, 'Ok',
                       actionListenerProc=self.btnOk_clicked )

        self.addButton('btnCancel', -2 - 45 - 5 , -10, 45, 15, 'Cancel',
                       actionListenerProc = self.btnCancel_clicked )

        # Variable Declaration
        self.sValue = None
        self.sObj = None
        self.aSectionList = []
        self.sGVariable = sVariable
        self.sGDisplayName = sDisplayName
        self.aItemList = []
        self.aComponentAdd = []
        self.aObjectList = []
        self.aListRepeatIn = []
        self.aVariableList = []
        # Call method to perform Enumration on Report Document
        EnumDocument(self.aItemList, self.aComponentAdd)
        # Perform checking that Field-1 and Field - 4 is available or not alos get Combobox
        # filled if condition is true
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        # Check weather Field-1 is available if not then exit from application
        self.sMyHost= ""

        if not docinfo.getUserFieldValue(3) == "" and not docinfo.getUserFieldValue(0) == "":
            self.sMyHost = docinfo.getUserFieldValue(0)
            self.count=0
            oParEnum = doc.getTextFields().createEnumeration()

            while oParEnum.hasMoreElements():
                oPar = oParEnum.nextElement()
                if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                    self.count += 1

            getList(self.aObjectList, self.sMyHost, self.count)
            cursor = doc.getCurrentController().getViewCursor()
            text = cursor.getText()
            tcur = text.createTextCursorByRange(cursor)

            self.aVariableList.extend( filter( lambda obj: obj[:obj.find(" ")] == "List", self.aObjectList ) )

            for i in range(len(self.aItemList)):
                try:
                    anItem = self.aItemList[i][1]
                    component = self.aComponentAdd[i]

                    if component == "Document":
                        sLVal = anItem[anItem.find(",'") + 2:anItem.find("')")]
                        self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == sLVal, self.aObjectList ) )

                    if tcur.TextSection:
                        getRecursiveSection(tcur.TextSection, self.aSectionList)
                        if component in self.aSectionList:
                            sLVal = anItem[anItem.find(",'") + 2:anItem.find("')")]
                            self.aVariableList.extend(
                                filter(lambda obj: obj[:obj.find("(")] == sLVal,
                                       self.aObjectList )
                            )

                    if tcur.TextTable:
                        if not component == "Document" \
                           and component[component.rfind(".") + 1:] == tcur.TextTable.Name:
                            VariableScope(tcur,
                                          self.aVariableList,
                                          self.aObjectList,
                                          self.aComponentAdd,
                                          self.aItemList,
                                          component )
                except Exception, exception :
                    logger.exception(exception)

            self.bModify = bFromModify

            if self.bModify == True:

                if sObject == "":
                    self.insVariable.setText("List of "+docinfo.getUserFieldValue(3))
                    self.insField.addItem("objects", self.getListBoxItemCount("lstFields"))
                    self.setEditText("txtName", sVariable)
                    self.setEditText("txtUName", sDisplayName)
                    self.sValue= "objects"
                else:
                    sItem=""
                    for anObject in self.aObjectList:
                        if anObject[:anObject.find("(")] == sObject:
                            sItem = anObject
                            self.insVariable.setText( sItem )

                    genTree(
			sItem[sItem.find("(")+1:sItem.find(")")],
			self.aListRepeatIn,
			self.insField,
			self.sMyHost,
			2,
			ending=['one2many', 'many2many'],
			recur=['one2many', 'many2many']
                    )

                    self.sValue = self.getListBoxItem("lstFields", self.aListRepeatIn.index(sFields))

            proxy_model = Object('ir.model')
            for var in self.aVariableList:
                if var[:8] != 'List of ':
                    domain = [('model', '=', var[var.find("(")+1:var.find(")")])]
                else:
                    domain = [('model', '=', var[8:])]
                self.model_res = proxy_model.select(domain, ['name', 'model'])
                if not isinstance(self.model_res, list):
                    if var[:8] != 'List of ':
                        self.insVariable.addItem(var[:var.find("(")+1] + self.model_res[0]['name'] + ")", self.insVariable.getItemCount())
                    else:
                        self.insVariable.addItem('List of ' + self.model_res[0]['name'], self.insVariable.getItemCount())
                else:
                    self.insVariable.addItem(var, self.insVariable.getItemCount())

            self.doModalDialog("lstFields", self.sValue)
        else:
            ErrorDialog("Please Select Appropriate module" ,"Create new report from: \nOpenERP -> Open a New Report")
            self.endExecute()

    def lstbox_selected(self, oItemEvent):
        sItem = self.getListBoxSelectedItem("lstFields")
        sMain = self.aListRepeatIn[self.getListBoxSelectedItemPos("lstFields")]

        if self.bModify == True:
            self.setEditText("txtName", self.sGVariable)
            self.setEditText("txtUName", self.sGDisplayName)
        else:
            self.setEditText("txtName", sMain[sMain.rfind("/")+1:])
            self.setEditText("txtUName","|-."+sItem[sItem.rfind("/")+1:]+".-|")

    def cmbVariable_selected(self, oItemEvent):
        if self.count > 0 :

            desktop = getDesktop()
            doc = desktop.getCurrentComponent()
            docinfo = doc.getDocumentInfo()
            self.removeListBoxItems("lstFields", 0, self.getListBoxItemCount("lstFields"))
            sItem = self.getComboBoxText("cmbVariable")
            for var in self.aVariableList:
                if var[:8] == 'List of ':
                    if var[:8] == sItem[:8]:
                        sItem = var
                elif var[:var.find("(")+1] == sItem[:sItem.find("(")+1]:
                    sItem = var
            self.aListRepeatIn = []

            data = ( sItem[sItem.rfind(" ") + 1:] == docinfo.getUserFieldValue(3) ) and docinfo.getUserFieldValue(3) or sItem[sItem.find("(")+1:sItem.find(")")]
            genTree( data, self.aListRepeatIn, self.insField, self.sMyHost, 2, ending=['one2many', 'many2many'], recur=['one2many', 'many2many'] )

            self.selectListBoxItemPos("lstFields", 0, True )

        else:
            sItem = self.getComboBoxText("cmbVariable")
            for var in self.aVariableList:
                if var[:8] == 'List of ' and var[:8] == sItem[:8]:
                    sItem = var
            if sItem.find(".") == -1:
                temp = sItem[sItem.rfind("x_"):]
            else:
                temp = sItem[sItem.rfind(".")+1:]
            self.setEditText("txtName", temp)
            self.setEditText("txtUName","|-."+temp+".-|")
            self.insField.addItem("objects", self.getListBoxItemCount("lstFields"))
            self.selectListBoxItemPos("lstFields", 0, True )

    def btnOk_clicked(self, oActionEvent):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        cursor = doc.getCurrentController().getViewCursor()
        selectedItem = self.getListBoxSelectedItem( "lstFields" )
        selectedItemPos = self.getListBoxSelectedItemPos( "lstFields" )
        txtName = self.getEditText( "txtName" )
        txtUName = self.getEditText( "txtUName" )
        if selectedItem != "" and txtName != "" and txtUName != "":
            sKey = u""+ txtUName
            if selectedItem == "objects":
                sValue = u"[[ repeatIn(" + selectedItem + ",'" + txtName + "') ]]"
            else:
                sObjName = self.getComboBoxText("cmbVariable")
                sObjName = sObjName[:sObjName.find("(")]
                sValue = u"[[ repeatIn(" + sObjName + self.aListRepeatIn[selectedItemPos].replace("/",".") + ",'" + txtName +"') ]]"

            if self.bModify == True:
                oCurObj = cursor.TextField
                oCurObj.Items = (sKey, sValue)
                oCurObj.update()
            else:
                oInputList = doc.createInstance("com.sun.star.text.TextField.DropDown")
                if self.getListBoxSelectedItem("lstFields") == "objects":
                    oInputList.Items = (sKey, sValue)
                    doc.Text.insertTextContent(cursor, oInputList, False)
                else:
                    sValue = u"[[ repeatIn(" + sObjName + self.aListRepeatIn[selectedItemPos].replace("/",".") + ",'" + txtName +"') ]]"
                    if cursor.TextTable is None:
                        oInputList.Items = (sKey, sValue)
                        doc.Text.insertTextContent(cursor, oInputList, False)
                    else:
                        oInputList.Items = (sKey, sValue)
                        widget = ( cursor.TextTable or selectedItem != 'objects' ) and cursor.TextTable.getCellByName( cursor.Cell.CellName ) or doc.Text
                        widget.insertTextContent(cursor, oInputList, False)
                self.endExecute()
        else:
            ErrorDialog("Please Fill appropriate data in Object Field or Name field \nor select perticular value from the list of fields")

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

class Fields(unohelper.Base, XJobExecutor, DBModalDialog ):
    @login_required
    def __init__(self, sVariable="", sFields="", sDisplayName="", bFromModify=False):
        DBModalDialog.__init__(self, 60, 50, 200, 225, "Field Builder")

        self.addFixedText("lblVariable", 27, 12, 60, 15, "Variable :")
        self.addComboBox("cmbVariable", 180-120-2, 10, 130, 15, True,
                         itemListenerProc=self.cmbVariable_selected)
        self.insVariable = self.getControl( "cmbVariable" )

        self.addFixedText("lblFields", 10, 32, 60, 15, "Variable Fields :")
        self.addComboListBox("lstFields", 180-120-2, 30, 130, 150, False, True,
                             itemListenerProc=self.lstbox_selected)
        self.insField = self.getControl( "lstFields" )

        self.addFixedText("lblUName", 8, 187, 60, 15, "Displayed name :")
        self.addEdit("txtUName", 180-120-2, 185, 130, 15,)

        self.addButton('btnOK', -5, -5, 45, 15, 'Ok',
                       actionListenerProc=self.btnOk_clicked )
        self.addButton('btnCancel', -5 - 45 - 5 , -5, 45, 15, 'Cancel',
                       actionListenerProc=self.btnCancel_clicked )

        self.sValue = None
        self.sObj = None
        self.aSectionList = []
        self.sGDisplayName = sDisplayName
        self.aItemList = []
        self.aComponentAdd = []
        self.aObjectList = []
        self.aListFields = []
        self.aVariableList = []
        EnumDocument(self.aItemList, self.aComponentAdd)
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        self.sMyHost= ""

        if not docinfo.getUserFieldValue(3) == "" and not docinfo.getUserFieldValue(0) == "":
            self.sMyHost = docinfo.getUserFieldValue(0)
            self.count = 0
            oParEnum = doc.getTextFields().createEnumeration()

            while oParEnum.hasMoreElements():
                oPar = oParEnum.nextElement()
                if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                    self.count += 1

            getList(self.aObjectList, self.sMyHost, self.count)
            cursor = doc.getCurrentController().getViewCursor()
            text = cursor.getText()
            tcur = text.createTextCursorByRange(cursor)

            self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == "Objects", self.aObjectList ) )

            for i in range(len(self.aItemList)):
                try:
                    anItem = self.aItemList[i][1]
                    component = self.aComponentAdd[i]

                    if component == "Document":
                        sLVal = anItem[anItem.find(",'") + 2:anItem.find("')")]
                        self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == sLVal, self.aObjectList ) )

                    if tcur.TextSection:
                        getRecursiveSection(tcur.TextSection, self.aSectionList)
                        if component in self.aSectionList:
                            sLVal = anItem[anItem.find(",'") + 2:anItem.find("')")]
                            self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == sLVal, self.aObjectList ) )

                    if tcur.TextTable:
                        if not component == "Document" and component[component.rfind(".")+1:] == tcur.TextTable.Name:
                            VariableScope(tcur, self.aVariableList, self.aObjectList, self.aComponentAdd, self.aItemList, component)
                except Exception, exception:
                    logger.exception(exception)

            self.bModify = bFromModify
            if self.bModify == True:
                sItem=""
                for anObject in self.aObjectList:
                    if anObject[:anObject.find("(")] == sVariable:
                        sItem = anObject
                        self.insVariable.setText(sItem)
                genTree(
                    sItem[sItem.find("(")+1:sItem.find(")")],
                    self.aListFields,
                    self.insField,
                    self.sMyHost,
                    2,
                    ending_excl=['one2many', 'many2one', 'many2many', 'reference'],
                    recur=['many2one']
                )
                self.sValue = self.getListBoxItem("lstFields", self.aListFields.index(sFields))

            proxy_model = Object('ir.model')
            for var in self.aVariableList:
                models = proxy_model.select([('model', '=', var[var.find("(")+1:var.find(")")])], ['name', 'model'])
                if not isinstance(models, list):
                    self.insVariable.addItem(var[:var.find("(")+1] + models[0]['name'] + ")",
                                             self.insVariable.getItemCount())
                else:
                    self.insVariable.addItem(var,
                                             self.insVariable.getItemCount())

            self.doModalDialog("lstFields", self.sValue)
        else:
            ErrorDialog("Please insert user define field Field-1 or Field-4","Just go to File->Properties->User Define \nField-1 Eg. %s \nOR \nField-4 Eg. account.invoice" % DEFAULT_SERVER_URL)
            self.endExecute()

    def lstbox_selected(self, oItemEvent):
        try:
            sItem = self.getComboBoxText("cmbVariable")
            for var in self.aVariableList:
                if var[:var.find("(")+1] == sItem[:sItem.find("(")+1]:
                    sItem = var
            sMain = self.aListFields[self.getListBoxSelectedItemPos("lstFields")]
            sObject = self.getRes(sItem[sItem.find("(")+1:-1], sMain[1:])

            proxy = Object(sObject)
            res = proxy.read(proxy.search([])[0])
            self.setEditText("txtUName", res[0][sMain[sMain.rfind("/")+1:]])
        except Exception, exception:
            logger.exception(exception)
            self.setEditText("txtUName","TTT")

        if self.bModify:
            self.setEditText("txtUName", self.sGDisplayName)

    def getRes(self, sObject, sVar):
        res = Object(sObject).fields_get()
        keys = sorted(res.keys())
        myval = None
        if not sVar.find("/") == -1:
            myval = sVar[:sVar.find("/")]
        else:
            myval = sVar
        if myval in keys:
            if (res[myval]['type'] in ['many2one']):
                sObject = res[myval]['relation']
                return self.getRes(res[myval]['relation'], sVar[sVar.find("/")+1:])
            else:
                return sObject

    def cmbVariable_selected(self, oItemEvent):
        if self.count > 0 :
            try:
                self.removeListBoxItems("lstFields", 0, self.getListBoxItemCount("lstFields"))
                self.aListFields = []
                tempItem = self.getComboBoxText("cmbVariable")
                for var in self.aVariableList:
                    if var[:var.find("(")] == tempItem[:tempItem.find("(")]:
                        sItem = var

                genTree(
                    sItem[sItem.find("(")+1:sItem.find(")")],
                    self.aListFields,
                    self.insField,
                    self.sMyHost,
                    2,
                    ending_excl=['one2many', 'many2one', 'many2many', 'reference'],
                    recur=['many2one']
                )
            except Exception, exception:
                logger.exception(exception)

    def btnOk_clicked( self, oActionEvent ):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        cursor = doc.getCurrentController().getViewCursor()
        for i in self.getListBoxSelectedItemsPos("lstFields"):
            itemSelected = self.aListFields[i]
            itemSelectedPos = i
            txtUName = self.getEditText("txtUName")
            sKey = u""+txtUName
            if itemSelected != "" and txtUName != "" and self.bModify == True :
                txtUName = self.sGDisplayName
                sKey = u""+txtUName
                txtUName = self.sGDisplayName
                oCurObj = cursor.TextField
                sObjName = self.insVariable.getText()
                sObjName = sObjName[:sObjName.find("(")]
                sValue = u"[[ " + sObjName + self.aListFields[itemSelectedPos].replace("/",".") + " ]]"
                oCurObj.Items = (sKey, sValue)
                oCurObj.update()
                self.endExecute()
            elif itemSelected != "" and txtUName != "" :
                oInputList = doc.createInstance("com.sun.star.text.TextField.DropDown")
                sObjName = self.getComboBoxText("cmbVariable")
                sObjName = sObjName[:sObjName.find("(")]
                widget = ( cursor.TextTable and cursor.TextTable.getCellByName( cursor.Cell.CellName ) or doc.Text )
                sValue = u"[[ " + sObjName + self.aListFields[itemSelectedPos].replace("/",".") + " ]]"
                oInputList.Items = (sKey, sValue)
                widget.insertTextContent(cursor, oInputList, False)
                self.endExecute()
            else:
                ErrorDialog("Please Fill appropriate data in Name field \nor select perticular value from the list of fields")

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

class Expression(unohelper.Base, XJobExecutor, DBModalDialog ):
    @login_required
    def __init__(self, sExpression="", sName="", bFromModify=False):
        DBModalDialog.__init__(self, 60, 50, 180, 65, "Expression Builder")
        self.addFixedText("lblExpression", 17 , 10, 35, 15, "Expression :")
        self.addEdit("txtExpression", -5, 5, 123, 15)
        self.addFixedText("lblName", 2, 30, 50, 15, "Displayed Name :")
        self.addEdit("txtName", -5, 25, 123, 15)
        self.addButton("btnOK", -5, -5, 40, 15, "OK",
                       actionListenerProc=self.btnOk_clicked )
        self.addButton("btnCancel", -5 - 40 -5, -5, 40, 15, "Cancel",
                       actionListenerProc=self.btnCancel_clicked )
        self.bModify = bFromModify
        if self.bModify == True:
            self.setEditText("txtExpression", sExpression)
            self.setEditText("txtName", sName)
        self.doModalDialog("", None)


    def btnOk_clicked( self, oActionEvent ):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        text = doc.Text
        cursor = doc.getCurrentController().getViewCursor()
        if self.bModify == True:
            oCurObj = cursor.TextField
            sKey = u""+self.getEditText("txtName")
            sValue = u"[[ " + self.getEditText("txtExpression") + " ]]"
            oCurObj.Items = (sKey, sValue)
            oCurObj.update()
            self.endExecute()
        else:
            oInputList = doc.createInstance("com.sun.star.text.TextField.DropDown")
            if self.getEditText("txtName") != "" and self.getEditText("txtExpression") != "":
                sKey = u""+self.getEditText("txtName")
                sValue = u"[[ " + self.getEditText("txtExpression") + " ]]"
                if cursor.TextTable is None:
                    oInputList.Items = (sKey, sValue)
                    text.insertTextContent(cursor, oInputList, False)
                else:
                    oTable = cursor.TextTable
                    oCurCell = cursor.Cell
                    tableText = oTable.getCellByName( oCurCell.CellName )
                    oInputList.Items = (sKey, sValue)
                    tableText.insertTextContent(cursor, oInputList, False)
                self.endExecute()
            else:
                ErrorDialog("Please Fill appropriate data in Name field or \nExpression field")

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

class modify(unohelper.Base, XJobExecutor ):
    def __init__( self, ctx ):
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION

        # Variable Declaration
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        self.oVC = doc.CurrentController.getViewCursor()
        if not docinfo.getUserFieldValue(0) == "":
            self.sMyHost = docinfo.getUserFieldValue(0)
        else:
            ErrorDialog(
		"Please insert user define field Field-1",
		"Just go to File->Properties->User Define \n"
		"Field-1 Eg. %s" % DEFAULT_SERVER_URL
            )
            exit(1)

        # Check weather Field-4 is available or not otherwise exit from application
        if not docinfo.getUserFieldValue(3) == "" and not docinfo.getUserFieldValue(0) == "":
            if self.oVC.TextField:
                self.oCurObj = self.oVC.TextField
                item = self.oCurObj.Items[0]
                kind, group1, group2 = self.getOperation(self.oCurObj.Items[1] )
                start_group1 = group1[:group1.find(".")]
                stop_group1 = group1[group1.find("."):].replace(".", "/")

                if kind == "field":
                    Fields( start_group1, stop_group1, item, True )
                elif kind == "expression":
                    Expression( group1, item, True )
                elif kind == "repeatIn":
                    RepeatIn( start_group1, group2, stop_group1, item, True )
            else:
                ErrorDialog(
                    "Please place your cursor at beginning of field \n"
                    "which you want to modify",""
                )

        else:
            ErrorDialog(
		"Please insert user define field Field-1 or Field-4",
		"Just go to File->Properties->User Define \n"
		"Field-1 Eg. %s \n"
		"OR \n"
		"Field-4 Eg. account.invoice" % DEFAULT_SERVER_URL
            )
            exit(1)

    def getOperation(self, str):
        #str = "[[ RepeatIn(objects, 'variable') ]]" #repeatIn
        #str = "[[ saleorder.partner_id.name ]]" # field
        #str = "[[ some thing complex ]]" # expression
        method1 = lambda x: (u'repeatIn', x.group(1), x.group(2))
        method2 = lambda x: (u'field', x.group(1), None)
        method3 = lambda x: (u'expression', x.group(1), None)
        regexes = [
            ('\\[\\[ *repeatIn\\( *(.+)*, *\'([a-zA-Z0-9_]+)\' *\\) *\\]\\]', method1),
            ('\\[\\[ *([a-zA-Z0-9_\.]+) *\\]\\]', method2),
            ('\\[\\[ *(.+) *\\]\\]', method3)
        ]
        for (rule, method) in regexes:
            res = re.match(rule, str)
            if res:
                return method(res)

class ServerParameter( unohelper.Base, XJobExecutor, DBModalDialog):
    def __init__(self, ctx):
        DBModalDialog.__init__(self, 60, 50, 160, 108, "Server Connection Parameter")
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION
        self.result = None
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()

        self.addFixedText("lblVariable", 2, 12, 35, 15, "Server URL")
        if docinfo.getUserFieldValue(0) == "":
            docinfo.setUserFieldValue(0, DEFAULT_SERVER_URL)
        self.addEdit("txtHost", -34, 9, 91, 15, docinfo.getUserFieldValue(0))
        self.addButton('btnChange', -2 , 9, 30, 15, 'Change',
                       actionListenerProc=self.btnChange_clicked )

        self.addFixedText("lblDatabaseName", 6, 31, 31, 15, "Database")

        #self.addFixedText("lblMsg", -2, 28, 123, 15)
        #self.addComboListBox("lstDatabase", -2, 28, 123, 15, True)
        #self.lstDatabase = self.getControl( "lstDatabase" )
        #self.selectListBoxItem( "lstDatabase", docinfo.getUserFieldValue(2), True )
        #self.setEnabled("lblMsg", False)

        self.addFixedText("lblLoginName", 17, 51, 20, 15, "Login")
        self.addEdit("txtLoginName", -2, 48, 123, 15, docinfo.getUserFieldValue(1))

        self.addFixedText("lblPassword", 6, 70, 31, 15, "Password")
        self.addEdit("txtPassword", -2, 67, 123, 15,)
        self.setEchoChar("txtPassword", 42)

        self.addButton('btnOK', -2 , -5, 60, 15, 'Connect',
                       actionListenerProc=self.btnOk_clicked)

        self.addButton('btnCancel', -2 - 60 - 5 , -5, 35, 15, 'Cancel',
                       actionListenerProc=self.btnCancel_clicked)
        sValue = None
        if docinfo.getUserFieldValue(0) != "":

            Configuration.url = docinfo.getUserFieldValue(0)
            connector = rpc_factory(Configuration.url)
            try:
                self.result = Database(connector).list()
            except Exception, exception:
                self.result = COULD_NOT_CONNECT_TO_SERVER
            logger.debug("result: %r", self.result)


            if self.result == COULD_NOT_CONNECT_TO_SERVER:
                control = self.getControl('lstDatabase')
                if control:
                    self.oDialogControl.removeControl(control)
                    self.oDialogModel.removeByName('lstDatabase')
                self.addEdit("lstDatabase", -2, 28, 123, 15)
                #sValue="Could not connect to the server!"
                #self.lstDatabase.addItem("Could not connect to the server!", 0)
            elif self.result == NO_AVAILABLE_DATABASE:
                sValue="No Database found !!!"
                self.lstDatabase.addItem("No Database found !!!", 0)
            else:
                control = self.getControl('lstDatabase')
                if control:
                    self.oDialogControl.removeControl(control)
                    self.oDialogModel.removeByName('lstDatabase')
                self.addComboListBox("lstDatabase", -2, 28, 123, 15, True)
                self.lstDatabase = self.getControl( "lstDatabase" )

                self.removeListBoxItems("lstDatabase", 0, self.getListBoxItemCount("lstDatabase"))
                for i in range(len(self.result)):
                    self.lstDatabase.addItem(self.result[i], i)
                sValue = Configuration.database
        self.doModalDialog("lstDatabase", sValue)

        #self.doModalDialog("lstDatabase", docinfo.getUserFieldValue(2))

    def btnOk_clicked(self, oActionEvent):
        sLogin = self.getEditText("txtLoginName")
        sPassword = self.getEditText("txtPassword")

        if self.result == COULD_NOT_CONNECT_TO_SERVER:
            sDatabase = self.getEditText("lstDatabase")
        else:
            sDatabase = self.getListBoxSelectedItem("lstDatabase")

        logger.debug('Configuration: %r', Configuration)
        connector = rpc_factory(self.getEditText('txtHost'))
        connection = Connection(connector, sDatabase, sLogin, sPassword)
        logger.debug('connection: %r', connection)

        logger.debug('user_id: %r', connection.user_id)
        if not connection.user_id:
            ErrorDialog("Connection Refused...","Please enter valid Login/Password")
            self.endExecute()

        groups = []
        try:
            proxy_group = Object('res.groups', connection)
            groups = proxy_group.select([('name', '=', 'OpenOfficeReportDesigner')],
                                        ['users'])
            logger.debug('groups: %r', groups)
            if not groups:
                ErrorDialog("""Group Not Found!!!  Create a group  named  \n\n"OpenOfficeReportDesigner" """,
                            "",
                            "Group Name Error")
                logger.warning('Create a group OpenOffice Report Designer using database %s',
                               sDatabase)
                self.endExecute()


            proxy_module = Object('ir.module.module', connection)
            try:
                module_ids = proxy_module.search([('name', '=', 'base_report_designer'),
                                                  ('state', '=', 'installed')])
                logger.debug('module_ids: %r', module_ids)
            except Exception, ex:
                ErrorDialog("""Your user has not the read right for the ir.module.module object""",
                            "",
                            "Bad Rights")
                logger.error('Please, add the rights on the ir.module.module object to your user %s', Configuration.username)
                raise

            if not module_ids:
                ErrorDialog("Please Install base_report_designer module", "", "Module Uninstalled Error")
                logger.warning('Please install the base_report_designer module in this database %s', sDatabase)
                self.endExecute()

        except Exception, exception:
            logger.exception(exception)
            exit(1)

        if connection.user_id not in groups[0]['users']:
            ErrorDialog("Connection Refused...","You don't have access to the Report Designer")
            logger.warning('You have not access to the Report Designer')
        else:
            desktop = getDesktop()
            doc = desktop.getCurrentComponent()
            docinfo = doc.getDocumentInfo()
            docinfo.setUserFieldValue(0, self.getEditText("txtHost"))
            docinfo.setUserFieldValue(1, self.getEditText("txtLoginName"))

            Configuration.password = self.getEditText("txtPassword")
            Configuration.username = self.getEditText('txtLoginName')
            Configuration.login_status = True
            Configuration.database = sDatabase
            Configuration.connector = rpc_factory(self.getEditText('txtHost'))
            Configuration.connection = connection

            #docinfo.setUserFieldValue(2, self.getListBoxSelectedItem("lstDatabase"))
            #docinfo.setUserFieldValue(3,"")

            ErrorDialog(" You can start creating your report \nin the current document.", "After Creating  sending to the server.","Message")
            logger.info('Successful login from %s using %s', sLogin, sDatabase)
        self.endExecute()

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

    def btnChange_clicked(self, oActionEvent):
        aVal = []
        url = self.getEditText("txtHost")
        Change(aVal, url)
        logger.debug("aVal: %r", aVal)
        logger.debug("type: %r", type(aVal))
        logger.debug('aVal[1]: %r', aVal[1])
        self.result = aVal
        if aVal[1] == COULD_NOT_CONNECT_TO_SERVER:
            self.getEditText("lstDatabase")
            #pass
        elif aVal[1] == NO_AVAILABLE_DATABASE:
            ErrorDialog(aVal[0],"")
        else:
            self.setEditText("txtHost", aVal[0])
            control = self.getControl('lstDatabase')
            if control:
                self.oDialogControl.removeControl(control)
                self.oDialogModel.removeByName('lstDatabase')
            self.addComboListBox("lstDatabase", -2, 28, 123, 15, True)
            control = self.getControl('lstDatabase')
            self.lstDatabase = control

            logger.debug('control: %r', control)
            logger.debug('dir: %r', dir(control))
            logger.debug('type: %r', type(control))
            self.removeListBoxItems("lstDatabase", 0,
                                    self.getListBoxItemCount("lstDatabase"))
            for i in range(len(aVal[1])):
                self.lstDatabase.addItem(aVal[1][i], i)

class Change(DBModalDialog):
    """
    This Dialog helps the user to change the parameters for the connection to the OpenERP Server
    """
    def __init__(self, aVal = None, sURL= ""):
        DBModalDialog.__init__(self, 60, 50, 120, 90, 'Connect to OpenERP Server')
        self.addFixedText("lblVariable", 38, 12, 60, 15, "Server")

        self.addEdit("txtHost", -2, 9, 60, 15, sURL[sURL.find("/")+2:sURL.rfind(":")])

        self.addFixedText("lblReportName", 45 , 31, 60, 15, "Port")
        self.addEdit("txtPort", -2, 28, 60, 15, sURL[sURL.rfind(":")+1:])

        self.addFixedText("lblLoginName", 2, 51, 60, 15, "Protocol Connection")

        self.addComboListBox("lstProtocol", -2, 48, 60, 15, True)
        self.lstProtocol = self.getControl( "lstProtocol" )
        protocols = PROTOCOLS.keys()
        for protocol in protocols:
            self.lstProtocol.addItem(protocol, self.lstProtocol.getItemCount() )

        self.addButton('btnOK', -2, -5, 30, 15, 'Ok',
                       actionListenerProc=self.btnOk_clicked)
        self.addButton('btnCancel', -2 - 30 - 5 , -5, 30, 15, 'Cancel',
                       actionListenerProc=self.btnCancel_clicked)

        self.aVal = aVal

        sValue = protocols[0]
        if sURL != "":
            sValue = protocols[PROTOCOLS.values().index(sURL[:sURL.find("/")+2])]

        self.doModalDialog( "lstProtocol", sValue)

    def btnOk_clicked(self, oActionEvent):
        url = "%s%s:%d" % (PROTOCOLS[self.getListBoxSelectedItem('lstProtocol')],
                            self.getEditText('txtHost'),
                            long(self.getEditText('txtPort')),)
        getDesktop().getCurrentComponent().getDocumentInfo().setUserFieldValue(0, url)

        # The value of result should be NO_AVAILABLE_DATABASE, COULD_NOT_CONNECT_TO_SERVER or a List of Database name
        Connection.url = url
        connector = rpc_factory(Connection.url)
        try:
            result = Database(connector).list()
        except Exception, ex:
            result = COULD_NOT_CONNECT_TO_SERVER

        information = url
        if result == NO_AVAILABLE_DATABASE:
            information = 'No Database Found !!!'
        self.aVal.append(information)
        self.aVal.append(result)
        self.endExecute()

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

#
# Start OpenOffice.org, listen for connections and open testing document
#
#
class NewReport(unohelper.Base, XJobExecutor, DBModalDialog):
    @login_required
    def __init__(self, ctx):
        DBModalDialog.__init__(self, 60, 50, 180, 115, "Open New Report")
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION
        self.addFixedText("lblModuleSelection", 2, 2, 60, 15, "Module Selection")
        self.addComboListBox("lstModule", -2, 13, 176, 80 , False)
        self.lstModule = self.getControl( "lstModule" )
        self.aModuleName = []

        proxy_model = Object('ir.model')
        models = proxy_model.select([], ['model', 'name'])
        models.sort(lambda x, y: cmp(x['name'], y['name']))

        for model in models:
            self.lstModule.addItem(model['name'], self.lstModule.getItemCount())
            self.aModuleName.append(model['model'])

        self.addButton('btnOK', -2 , -5, 70, 15, 'Use Module in Report',
                       actionListenerProc=self.btnOk_clicked)
        self.addButton('btnCancel', -2 - 70 - 5 , -5, 35, 15, 'Cancel',
                       actionListenerProc=self.btnCancel_clicked )
        self.doModalDialog("", None)

    def btnOk_clicked(self, oActionEvent):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        docinfo.setUserFieldValue(3, self.aModuleName[self.lstModule.getSelectedItemPos()])
        logger.info('Module use in creating a report %s using database %s',
                    self.aModuleName[self.lstModule.getSelectedItemPos()],
                    Configuration.database)
        self.endExecute()

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

class ModifyExistingReport(unohelper.Base, XJobExecutor, DBModalDialog):
    @login_required
    def __init__(self, ctx):
        DBModalDialog.__init__(self, 60, 50, 180, 120, "Modify Existing Report")
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION
        self.addFixedText("lblReport", 2, 3, 60, 15, "Report Selection")
        self.addComboListBox("lstReport", -1, 15, 178, 80 , False )
        self.lstReport = self.getControl( "lstReport" )
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        self.hostname = docinfo.getUserFieldValue(0)

        proxy_module = Object('ir.module.module')
        module_ids = proxy_module.search([('name', '=', 'base_report_designer'),
                                          ('state', '=', 'installed')])

        if not module_ids:
            ErrorDialog("Please Install base_report_designer module", "", "Module Uninstalled Error")
            exit(1)

        self.reports = Object('ir.actions.report.xml').select([('report_xsl', '=', False),
                                                               ('report_xml', '=', False)],
                                                              ['name', 'report_name', 'model'])
        self.report_with_id = []

        proxy_model = Object('ir.model')
        for report in self.reports:
            if report['name'] != "":
                models = proxy_model.select([('model', '=', report['model'])], ['name', 'model'])
                if not isinstance(models, list):
                    name = models[0]['name'] + " - " + report['name']
                else:
                    name = report['name'] + " - " + report['model']
                self.report_with_id.append( (report['id'], name, report['model'] ) )

        self.report_with_id.sort( lambda x, y: cmp( x[1], y[1] ) )

        for id, report_name, model_name in self.report_with_id:
            self.lstReport.addItem( report_name, self.lstReport.getItemCount() )

        self.addButton('btnSave', 10, -5, 50, 15, 'Open Report',
                       actionListenerProc=self.btnOk_clicked)
        self.addButton('btnCancel', -10 , -5, 50, 15, 'Cancel',
                       actionListenerProc=self.btnCancel_clicked )
        self.addButton('btnDelete', 15 -80 , -5, 50, 15, 'Delete Report',
                       actionListenerProc=self.btnDelete_clicked)
        self.doModalDialog("lstReport", self.report_with_id[0][1] )

    def btnOk_clicked(self, oActionEvent):
        try:
            desktop = getDesktop()

            selectedItemPos = self.getListBoxSelectedItemPos( "lstReport" )
            id = self.report_with_id[ selectedItemPos ][0]

            res = Object('ir.actions.report.xml').report_get(id)

            if res['file_type'] in ['sxw', 'odt'] :
                file_type = res['file_type']
            else :
                file_type = 'sxw'

            fp_name = tempfile.mktemp('.'+file_type)
            fp_name1="r"+fp_name
            fp_path = os.path.join(fp_name1).replace("\\","/")
            fp_win = fp_path[1:]

            filename = ( os.name == 'nt' and fp_win or fp_name )
            if res['report_sxw_content']:
                write_data_to_file( filename, base64.decodestring(res['report_sxw_content']))
            url = "file:///%s" % filename

            arr = Array(makePropertyValue("MediaType","application/vnd.sun.xml.writer"),)
            oDoc2 = desktop.loadComponentFromURL(url, "openerp", 55, arr)
            docinfo2=oDoc2.getDocumentInfo()
            docinfo2.setUserFieldValue(0, self.hostname)
            docinfo2.setUserFieldValue(1, Configuration.password)
            docinfo2.setUserFieldValue(2, id)
            docinfo2.setUserFieldValue(3, self.report_with_id[selectedItemPos][2])

            oParEnum = oDoc2.getTextFields().createEnumeration()
            while oParEnum.hasMoreElements():
                oPar = oParEnum.nextElement()
                if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                    oPar.SelectedItem = oPar.Items[0]
                    oPar.update()
            if oDoc2.isModified():
                if oDoc2.hasLocation() and not oDoc2.isReadonly():
                    oDoc2.store()

            ErrorDialog("Download is Completed","Your file has been placed here :\n"+ fp_name,"Download Message")
            logger.info('Successful download report %s using database %s',
                       self.report_with_id[selectedItemPos][2],
                       Configuration.database)
        except Exception, exception:
            ErrorDialog("Report has not been downloaded", "Report: %s\nDetails: %s" % ( fp_name, str(exception) ),"Download Message")
            logger.exception(exception)

        self.endExecute()

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()

    def btnDelete_clicked( self, oActionEvent ):
        selectedItemPos = self.getListBoxSelectedItemPos( "lstReport" )
        name = self.getListBoxSelectedItem ("lstReport")
        report_id = self.report_with_id[ selectedItemPos ][0]

        temp = Object('ir.actions.report.xml').unlink([report_id])

        proxy_values = Object('ir.values')
        value_ids = proxy_values.search([('value', '=', 'ir.actions.report.xml,%d' % report_id)])

        if value_ids:
            proxy_values.unlink(value_ids)

        if temp:
            ErrorDialog("Report","Report has been Delete:\n "+name,"Message")
            logger.info('successful delete report %s using database %s', name, Configuration.database)
        else:
            ErrorDialog("Report","Report has not Delete:\n"+name," Message")
        self.endExecute()

#
#
class SendtoServer(unohelper.Base, XJobExecutor):
    Kind = {
        'PDF' : 'pdf',
        'OpenOffice': 'sxw',
        'HTML' : 'html'
    }

    @login_required
    def __init__(self, ctx):
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION

        desktop = getDesktop()
        oDoc2 = desktop.getCurrentComponent()
        docinfo = oDoc2.getDocumentInfo()

        if not Object('ir.module.module').search([('name', '=', 'base_report_designer'),('state', '=', 'installed')]):
            ErrorDialog("Please Install base_report_designer module", "", "Module Uninstalled Error")
            exit(1)

        report_name = ""
        name=""
        if docinfo.getUserFieldValue(2) != "" :
            try:
                report = Object('ir.actions.report.xml').read([int(docinfo.getUserFieldValue(2))], ['name', 'report_name'])[0]
                name, report_name = report['name'], report['report_name']
            except Exception, exception:
                logger.exception(exception)

        elif docinfo.getUserFieldValue(3) != "":
            name = ""
            result =  "rnd"
            for i in range(5):
                result = result + random.choice('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890')

            report_name = docinfo.getUserFieldValue(3) + "." + result
        else:
            ErrorDialog("Please select appropriate module...","Note: use OpenERP Report -> Open a new Report", "Module selection ERROR")
            exit(1)

        self.win = DBModalDialog(60, 50, 180, 100, "Send To Server")
        self.win.addFixedText("lblName", 10 , 9, 40, 15, "Report Name :")
        self.win.addEdit("txtName", -5, 5, 123, 15, name)
        self.win.addFixedText("lblReportName", 2, 30, 50, 15, "Technical Name :")
        self.win.addEdit("txtReportName", -5, 25, 123, 15, report_name)
        self.win.addCheckBox("chkHeader", 51, 45, 70, 15, "Corporate Header")
        self.win.addFixedText("lblResourceType", 2, 60, 50, 15, "Select Rpt. Type :")
        self.win.addComboListBox("lstResourceType", -5, 58, 123, 15, True,
                                 itemListenerProc=self.lstbox_selected)
        self.lstResourceType = self.win.getControl( "lstResourceType" )
        self.txtReportName = self.win.getControl( "txtReportName" )
        self.txtReportName.Enable = False
        for kind in self.Kind.keys():
            self.lstResourceType.addItem( kind, self.lstResourceType.getItemCount() )

        self.win.addButton( "btnSend", -5, -5, 80, 15, "Send Report to Server",
                           actionListenerProc=self.btnOk_clicked)
        self.win.addButton( "btnCancel", -5 - 80 -5, -5, 40, 15, "Cancel",
                           actionListenerProc=self.btnCancel_clicked)

        self.win.doModalDialog("lstResourceType", self.Kind.keys()[0])

    def lstbox_selected(self, oItemEvent):
        pass

    def btnCancel_clicked( self, oActionEvent ):
        self.win.endExecute()

    def btnOk_clicked(self, oActionEvent):
        proxy_report = Object('ir.actions.report.xml')
        if self.win.getEditText("txtName") != "" and self.win.getEditText("txtReportName") != "":
            desktop = getDesktop()
            oDoc2 = desktop.getCurrentComponent()
            docinfo = oDoc2.getDocumentInfo()
            self.getInverseFieldsRecord(1)
            fp_name = tempfile.mktemp('.'+"sxw")
            if not oDoc2.hasLocation():
                oDoc2.storeAsURL("file://"+fp_name, Array(makePropertyValue("MediaType","application/vnd.sun.xml.writer"),))

            if docinfo.getUserFieldValue(2) == "":
                name = self.win.getEditText("txtName"),
                try:
                    record_ids = proxy_report.search([('name', '=', name)])
                    if not record_ids:
                        id = self.getID()
                        docinfo.setUserFieldValue(2, id)
                        rec = {
                            'name': self.win.getEditText("txtReportName"),
                            'key': 'action',
                            'model': docinfo.getUserFieldValue(3),
                            'value': 'ir.actions.report.xml,' + str(id),
                            'key2': 'client_print_multi',
                            'object': True
                        }
                        Object('ir.values').create(rec)
                    else :
                        ErrorDialog(" Report Name is all ready given !!!\n\n\n Please specify other Name","","Report Name")
                        logger.warning('Report name all ready given database %s', Configuration.database)
                        self.win.endExecute()
                except Exception, exception:
                    logger.exception(exception)
            else:

                record_id = docinfo.getUserFieldValue(2)
                proxy_values = Object('ir.values')
                proxy_values.write(
                    proxy_values.search([('value', '=', 'ir.actions.report.xml,%d' % long(record_id))]),
                    {'name': self.win.getEditText('txtReportName')}
                )
            oDoc2.store()
            data = read_data_from_file( get_absolute_file_path( oDoc2.getURL()[7:] ) )
            self.getInverseFieldsRecord(0)

            file_type = oDoc2.getURL()[7:].split(".")[-1]
            proxy_report.upload_report(int(docinfo.getUserFieldValue(2)),
                                       base64.encodestring(data),
                                       file_type,
                                       {}
                                      )
            params = {
                'name': self.win.getEditText("txtName"),
                'model': docinfo.getUserFieldValue(3),
                'report_name': self.win.getEditText("txtReportName"),
                'header': (self.win.getCheckBoxState("chkHeader") != 0),
                'report_type': self.Kind[self.win.getListBoxSelectedItem("lstResourceType")],
            }
            if self.win.getListBoxSelectedItem("lstResourceType") == 'OpenOffice':
                params['report_type']=file_type

            proxy_report.write(int(docinfo.getUserFieldValue(2)), params)
            logger.info('Report %s successfully send using %s', params['name'], Configuration.database)
            self.win.endExecute()
        else:
            ErrorDialog("Either Report Name or Technical Name is blank !!!\nPlease specify appropriate Name","","Blank Field ERROR")
            logger.info('Either Report name or technical name is blank')
            self.win.endExecute()

    def getID(self):
        docinfo = getDesktop().getCurrentComponent().getDocumentInfo()

        params = {
            'name': self.win.getEditText("txtName"),
            'model': docinfo.getUserFieldValue(3),
            'report_name': self.win.getEditText('txtReportName')
        }

        return Object('ir.actions.report.xml').create(params)

    def getInverseFieldsRecord(self, nVal):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        oParEnum = doc.getTextFields().createEnumeration()
        while oParEnum.hasMoreElements():
            oPar = oParEnum.nextElement()
            if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                oPar.SelectedItem = oPar.Items[nVal]
                if nVal == 0:
                    oPar.update()


class About(unohelper.Base, XJobExecutor, DBModalDialog):
    def __init__(self, ctx):
        DBModalDialog.__init__(self, 60, 50, 175, 115, ".:: About OpenERP Report Designer ::.")
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION

        fdBigFont = createUnoStruct("com.sun.star.awt.FontDescriptor")
        fdBigFont.Width = 20
        fdBigFont.Height = 25
        fdBigFont.Weight = 120
        fdBigFont.Family = 3

        oLabelTitle1 = self.addFixedText("lblTitle1", 1, 1, 35, 30)
        oLabelTitle1.Model.TextColor = 16056320
        oLabelTitle1.Model.FontDescriptor = fdBigFont
        oLabelTitle1.Model.FontRelief = 1
        oLabelTitle1.Text = "Open"

        oLabelTitle2 = self.addFixedText("lblTitle2", 35, 1, 30, 30)
        oLabelTitle2.Model.TextColor = 1
        oLabelTitle2.Model.FontDescriptor = fdBigFont
        oLabelTitle2.Model.FontRelief = 1
        oLabelTitle2.Text = "ERP"

        oLabelProdDesc = self.addFixedText("lblProdDesc", 1, 30, 173, 75)
        oLabelProdDesc.Model.TextColor = 1
        fdBigFont.Width = 10
        fdBigFont.Height = 11
        fdBigFont.Weight = 76
        oLabelProdDesc.Model.FontDescriptor = fdBigFont
        oLabelProdDesc.Model.Align = 1
        oLabelProdDesc.Model.FontRelief = 1
        oLabelProdDesc.Model.MultiLine = True
        oLabelProdDesc.Text = "This  package  helps  you  to  create  or  modify\n" \
                            "reports  in  OpenERP.  Once  connected  to  the\n" \
                            "server, you can design your template of reports\n" \
                            "using fields  and expressions  and  browsing the\n" \
                            "complete structure of OpenERP object database."

        oLabelFooter = self.addFixedText("lblFooter", -1, -1, 173, 25)
        oLabelFooter.Model.TextColor = 255
        #oLabelFooter.Model.BackgroundColor = 1
        oLabelFooter.Model.Border = 2
        oLabelFooter.Model.BorderColor = 255
        fdBigFont.Width = 8
        fdBigFont.Height = 9
        fdBigFont.Weight = 100
        oLabelFooter.Model.FontDescriptor = fdBigFont
        oLabelFooter.Model.Align = 1
        oLabelFooter.Model.FontRelief = 1
        oLabelFooter.Model.MultiLine = True
        sMessage = "OpenERP Report Designer v1.0 \n " \
                    "Copyright 2007-TODAY Tiny sprl \n" \
                    "This product is free software, under the GPL licence."
        oLabelFooter.Text = sMessage

        self.doModalDialog("", None)

class ConvertBracesToField( unohelper.Base, XJobExecutor ):

    @login_required
    def __init__(self, ctx):

        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION

        self.aReportSyntex = []
        self.getBraces(self.aReportSyntex)

        self.setValue()


    def setValue(self):

        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        regexes = [
            ['[a-zA-Z0-9_]+\.[a-zA-Z0-9_.]+',"Field"],
            ['\\[\\[ *repeatIn\\( *([a-zA-Z0-9_\.]+), *\'([a-zA-Z0-9_]+)\' *\\) *\\]\\]', "RepeatIn"],
            ['\\[\\[ *([a-zA-Z0-9_\.]+) *\\]\\]', "Field"]
            # ['\\[\\[ ([a-zA-Z0-9_]+\.[a-zA-Z1-9]) \\]\\]',"Field"],
            # ['\\[\\[ [a-zA-Z0-9_\.]+ and ([a-zA-Z0-9_\.]+) or .+? \\]\\]',"Field"],
            # ['\\[\\[ ([a-zA-Z0-9_\.]+) or .+? \\]\\]',"Field"],
            # ['\\[\\[ ([a-zA-Z0-9_\.]+) and .+? \\]\\]',"Field"],
            # ['\\[\\[ .+? or ([a-zA-Z0-9_\.]+) \\]\\]',"Field"],
            # ['\\[\\[ (.+?) and ([a-zA-Z0-9_\.]+) \\]\\]',"Field"],
            # ['\\[\\[ .+? % ([a-zA-Z0-9_\.]+) \\]\\]',"Field"]
        ]
        oRepeatInObjects = []
        saRepeatInList = []
        sHost = docinfo.getUserFieldValue(0)
        nCount = 0
        oParEnum = doc.getTextFields().createEnumeration()
        while oParEnum.hasMoreElements():
            oPar = oParEnum.nextElement()
            nCount += 1
        getList(oRepeatInObjects, sHost, nCount)
        for ro in oRepeatInObjects:
            if ro.find("(") != -1:
                saRepeatInList.append( [ ro[:ro.find("(")], ro[ro.find("(")+1:ro.find(")")] ])
        try:
            oParEnum = doc.getTextFields().createEnumeration()
            while oParEnum.hasMoreElements():
                oPar = oParEnum.nextElement()
                if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                    for reg in regexes:
                        res = re.findall(reg[0], oPar.Items[1])
                        if len(res) != 0:
                            if res[0][0] == "objects":
                                sTemp = docinfo.getUserFieldValue(3)
                                sTemp = "|-." + sTemp[sTemp.rfind(".")+1:] + ".-|"
                                oPar.Items=(sTemp.encode("utf-8"),oPar.Items[1].replace(' ',""))
                                oPar.update()
                            elif type(res[0]) != type(u''):

                                sObject = self.getRes(docinfo.getUserFieldValue(3), res[0][0][res[0][0].find(".")+1:].replace(".","/"))
                                r = Object(docinfo.getUserFieldValue(3)).fields_get()
                                sExpr="|-." + r[res[0][0][res[0][0].rfind(".")+1:]]["string"] + ".-|"
                                oPar.Items=(sExpr.encode("utf-8"),oPar.Items[1].replace(' ',""))
                                oPar.update()
                            else:

                                obj = None
                                for rl in saRepeatInList:
                                    if rl[0] == res[0][:res[0].find(".")]:
                                        obj = rl[1]
                                try:
                                    sObject = self.getRes(obj, res[0][res[0].find(".")+1:].replace(".","/"))
                                    r = Object(sObject).read([1])
                                except Exception, exception:
                                    r = "TTT"
                                    logger.exception(exception)

                                if len(r):
                                    if r != "TTT":
                                        if len(res)>1:
                                            sExpr=""
                                            if reg[1] == 'Field':
                                                for ires in res:
                                                    try:
                                                        sExpr = r[0][ires[ires.rfind(".")+1:]]
                                                        break
                                                    except Exception, exception:
                                                        logger.exception(exception)
                                                try:
                                                    oPar.Items=(sExpr.encode("utf-8"), oPar.Items[1])
                                                    oPar.update()
                                                except Exception, exception:
                                                    oPar.Items=(str(sExpr), oPar.Items[1])
                                                    oPar.update()
                                                    logger.exception(exception)

                                        else:
                                            sExpr = r[0][res[0][res[0].rfind(".")+1:]]
                                            try:

                                                if sExpr:
                                                    oPar.Items=(sExpr.encode("utf-8"), oPar.Items[1])
                                                    oPar.update()
                                                else:
                                                    oPar.Items=(u"/", oPar.Items[1])
                                                    oPar.update()
                                            except Exception, exception:
                                                oPar.Items=(str(sExpr), oPar.Items[1])
                                                oPar.update()
                                                logger.exception(exception)

                                    else:
                                        oPar.Items=(u""+r, oPar.Items[1])
                                        oPar.update()
                                else:
                                    oPar.Items=(u"TTT", oPar.Items[1])
                                    oPar.update()
        except Exception, exception:
            logger.exception(exception)

    def getRes(self, sObject, sVar):
        res = Object(sObject).fields_get()
        keys = sorted(res.keys())
        myval = None
        if not sVar.find("/") == -1:
            myval = sVar[:sVar.find("/")]
        else:
            myval = sVar
        for key in keys:
            if (res[key]['type'] in ['many2one']) and key == myval:
                sObject = self.getRes(res[myval]['relation'], sVar[sVar.find("/")+1:])
        return sObject

    def getBraces(self, aReportSyntex=[]):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        aRes = []

        try:
            regexes = [
                ['\\[\\[ *repeatIn\\( *([a-zA-Z0-9_\.]+), *\'([a-zA-Z0-9_]+)\' *\\) *\\]\\]', "RepeatIn"],
                ['\\[\\[ *([a-zA-Z0-9_\.]+) *\\]\\]', "Field"],
                ['\\[\\[ *.+? *\\]\\]', "Expression"]
            ]

            search = doc.createSearchDescriptor()
            search.SearchRegularExpression = True

            for reg in regexes:
                search.SearchString = reg[0]
                found = doc.findFirst( search )
                while found:
                    res = re.findall(reg[0], found.String)

                    if found.String not in [r[0] for r in aReportSyntex] and len(res) == 1 :
                        text = found.getText()
                        oInputList = doc.createInstance("com.sun.star.text.TextField.DropDown")
                        if reg[1] != "Expression":
                            oInputList.Items=(u""+found.String, u""+found.String)
                        else:
                            oInputList.Items=(u"?", u""+found.String)
                        aReportSyntex.append([oInputList, reg[1]])
                        text.insertTextContent(found, oInputList, False)
                        found.String =""

                    else:
                        aRes.append([res, reg[1]])
                        found = doc.findNext(found.End, search)
            search = doc.createSearchDescriptor()
            search.SearchRegularExpression = False

            for res in aRes:
                for r in res[0]:
                    search.SearchString = r
                    found = doc.findFirst(search)
                    while found:

                        text = found.getText()

                        oInputList = doc.createInstance("com.sun.star.text.TextField.DropDown")
                        if res[1] != "Expression":
                            oInputList.Items=(u""+found.String, u""+found.String)
                        else:
                            oInputList.Items=(u"?", u""+found.String)
                        aReportSyntex.append([oInputList, res[1]])
                        text.insertTextContent(found, oInputList, False)
                        found.String =""
                        found = doc.findNext(found.End, search)
        except Exception, exception:
            logger.exception(exception)

class ConvertFieldsToBraces( unohelper.Base, XJobExecutor ):
    @login_required
    def __init__(self, ctx):
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION
        self.aReportSyntex = []
        self.getFields()

    def getFields(self):
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()

        oParEnum = doc.getTextFields().createEnumeration()
        while oParEnum.hasMoreElements():
            oPar = oParEnum.nextElement()
            if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                oPar.getAnchor().Text.insertString(oPar.getAnchor(),oPar.Items[1], False)
                oPar.dispose()

class ExportToRML( unohelper.Base, XJobExecutor ):
    @login_required
    def __init__(self, ctx):
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION

        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()

        # Read Data from sxw file
        tmpsxw = tempfile.mktemp('.'+"sxw")

        if not doc.hasLocation():
            mytype = Array(makePropertyValue("MediaType","application/vnd.sun.xml.writer"),)
            doc.storeAsURL("file://"+tmpsxw, mytype)
        data = read_data_from_file( get_absolute_file_path( doc.getURL()[7:] ) )
        file_type = doc.getURL()[7:].split(".")[-1]
        if docinfo.getUserFieldValue(2) == "":
            ErrorDialog("Please Save this file on server","Use Send To Server Option in OpenERP Report Menu","Error")
            exit(1)
        filename = self.GetAFileName()
        if not filename:
            exit(1)

        try:
            res = Object('ir.actions.report.xml').sxwtorml(base64.encodestring(data),file_type)
            if res['report_rml_content']:
                write_data_to_file( get_absolute_file_path( filename[7:] ), res['report_rml_content'] )
        except Exception, exception:
            logger.exception(exception)
            ErrorDialog("Can't save the file to the hard drive.", "Exception: %s" % exception, "Error" )

    def GetAFileName(self):
        sFilePickerArgs = Array(10)
        oFileDialog = createUnoService("com.sun.star.ui.dialogs.FilePicker")
        oFileDialog.initialize(sFilePickerArgs)
        oFileDialog.appendFilter("OpenERP Report File Save To ....","*.rml")

        f_path = "OpenERP-"+ os.path.basename( tempfile.mktemp("","") ) + ".rml"
        initPath = tempfile.gettempdir()
        oUcb = createUnoService("com.sun.star.ucb.SimpleFileAccess")
        if oUcb.exists(initPath):
            oFileDialog.setDisplayDirectory('file://' + ( os.name == 'nt' and '/' or '' ) + initPath )

        oFileDialog.setDefaultName(f_path )

        sPath = oFileDialog.execute() == 1 and oFileDialog.Files[0] or None
        oFileDialog.dispose()
        return sPath

class AddLang(unohelper.Base, XJobExecutor, DBModalDialog):
    @login_required
    def __init__(self, sVariable="", sFields="", sDisplayName="", bFromModify=False):
        DBModalDialog.__init__(self, 60, 50, 180, 225, "Set Lang Builder")

        self.addFixedText("lblVariable", 27, 12, 60, 15, "Variable :")
        self.addComboBox("cmbVariable", 180-120-2, 10, 120, 15, True,
                         itemListenerProc = self.cmbVariable_selected)
        self.insVariable = self.getControl( "cmbVariable" )

        self.addFixedText("lblFields", 10, 32, 60, 15, "Variable Fields :")
        self.addComboListBox("lstFields", 180-120-2, 30, 120, 150, False,
                             itemListenerProc=self.lstbox_selected)
        self.insField = self.getControl( "lstFields" )
        self.addFixedText("lblUName", 8, 187, 60, 15, "Displayed name :")
        self.addEdit("txtUName", 180-120-2, 185, 120, 15,)

        self.addButton('btnOK', -5 , -5, 45, 15, 'Ok',
                       actionListenerProc = self.btnOk_clicked )
        self.addButton('btnCancel', -5 - 45 - 5 , -5, 45, 15, 'Cancel',
                       actionListenerProc = self.btnCancel_clicked )
        self.sValue = None
        self.sObj = None
        self.aSectionList = []
        self.sGDisplayName = sDisplayName
        self.aItemList = []
        self.aComponentAdd = []
        self.aObjectList = []
        self.aListFields = []
        self.aVariableList = []
        EnumDocument(self.aItemList, self.aComponentAdd)
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        docinfo = doc.getDocumentInfo()
        self.sMyHost= ""

        if not docinfo.getUserFieldValue(3) == "" and not docinfo.getUserFieldValue(0) == "":
            self.sMyHost = docinfo.getUserFieldValue(0)
            self.count=0
            oParEnum = doc.getTextFields().createEnumeration()
            while oParEnum.hasMoreElements():
                oPar = oParEnum.nextElement()
                if oPar.supportsService("com.sun.star.text.TextField.DropDown"):
                    self.count += 1
            getList(self.aObjectList, self.sMyHost, self.count)
            cursor = doc.getCurrentController().getViewCursor()
            text = cursor.getText()
            tcur = text.createTextCursorByRange(cursor)

            self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == "Objects", self.aObjectList ) )

            for i in range(len(self.aItemList)):
                anItem = self.aItemList[i][1]
                component = self.aComponentAdd[i]

                if component == "Document":
                    sLVal = anItem[anItem.find(",'") + 2:anItem.find("')")]
                    self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == sLVal, self.aObjectList ) )

                if tcur.TextSection:
                    getRecursiveSection(tcur.TextSection, self.aSectionList)
                    if component in self.aSectionList:
                        sLVal = anItem[anItem.find(",'") + 2:anItem.find("')")]
                        self.aVariableList.extend( filter( lambda obj: obj[:obj.find("(")] == sLVal, self.aObjectList ) )

                if tcur.TextTable:
                    if not component == "Document" and component[component.rfind(".") + 1:] == tcur.TextTable.Name:
                        VariableScope(tcur, self.insVariable, self.aObjectList, self.aComponentAdd, self.aItemList, component)

            self.bModify = bFromModify
            if self.bModify == True:
                sItem=""
                for anObject in self.aObjectList:
                    if anObject[:anObject.find("(")] == sVariable:
                        sItem = anObject
                        self.insVariable.setText( sItem )

                genTree(sItem[sItem.find("(")+1:sItem.find(")")], self.aListFields, self.insField, self.sMyHost, 2,ending_excl=['one2many', 'many2one', 'many2many', 'reference'], recur=['many2one'])
                self.sValue = self.getListBoxItem("lstFields", self.aListFields.index(sFields))

            proxy_model = Object('ir.model')
            for var in self.aVariableList:
                models = proxy_model.select([('model', '=', var[var.find("(")+1:var.find(")")])], ['name', 'model'])
                if not isinstance(models, list):
                    self.insVariable.addItem(var[:var.find("(")+1] + models[0]['name'] + ")", self.insVariable.getItemCount())
                else:
                    self.insVariable.addItem(var, self.insVariable.getItemCount())

            self.doModalDialog("lstFields", self.sValue)
        else:
            ErrorDialog("Please insert user define field Field-1 or Field-4","Just go to File->Properties->User Define \nField-1 Eg. http://localhost:8069 \nOR \nField-4 Eg. account.invoice")
            self.endExecute()

    def lstbox_selected(self, oItemEvent):
        try:
            sItem = self.getComboBoxText("cmbVariable")
            for var in self.aVariableList:
                if var[:var.find("(")+1] == sItem[:sItem.find("(")+1]:
                    sItem = var
            sMain = self.aListFields[self.getListBoxSelectedItemPos("lstFields")]
            t = sMain.rfind('/lang')
            if t!=-1:
                sObject = self.getRes(sItem[sItem.find("(")+1:-1], sMain[1:])
                proxy = Object(sObject)
                res = proxy.read([proxy.search([])[0]])
                self.setEditText("txtUName", res[0][sMain[sMain.rfind("/")+1:]])
            else:
                ErrorDialog("Please select the Language Field")

        except Exception, exception:
            logger.exception(exception)
            self.setEditText("txtUName","TTT")

        if self.bModify:
            self.setEditText("txtUName", self.sGDisplayName)

    def getRes(self, sObject, sVar):
        res = Object(sObject).fields_get()
        keys = sorted(res.keys())
        myval = None
        if not sVar.find("/") == -1:
            myval = sVar[:sVar.find("/")]
        else:
            myval = sVar
        if myval in keys:
            if (res[myval]['type'] in ['many2one']):
                sObject = res[myval]['relation']
                return self.getRes(res[myval]['relation'], sVar[sVar.find("/")+1:])
            else:
                return sObject

    def cmbVariable_selected(self, oItemEvent):
        if self.count > 0 :
            try:
                self.removeListBoxItems("lstFields", 0, self.getListBoxItemCount("lstFields"))
                self.aListFields = []
                tempItem = self.getComboBoxText("cmbVariable")
                for var in self.aVariableList:
                    if var[:var.find("(")] == tempItem[:tempItem.find("(")]:
                        sItem = var

                genTree(
                    sItem[ sItem.find("(") + 1:sItem.find(")")],
                    self.aListFields,
                    self.insField,
                    self.sMyHost,
                    2,
                    ending_excl=['one2many', 'many2one', 'many2many', 'reference'],
                    recur=['many2one']
                )

            except Exception, exception:
                logger.exception(exception)

    def btnOk_clicked( self, oActionEvent ):
        self.bOkay = True
        desktop = getDesktop()
        doc = desktop.getCurrentComponent()
        cursor = doc.getCurrentController().getViewCursor()

        itemSelected = self.getListBoxSelectedItem( "lstFields" )
        itemSelectedPos = self.getListBoxSelectedItemPos( "lstFields" )
        txtUName = self.getEditText("txtUName")
        sKey = u""+ txtUName

        if itemSelected != "" and txtUName != "" and self.bModify == True :
            oCurObj = cursor.TextField
            sObjName = self.insVariable.getText()
            sObjName = sObjName[:sObjName.find("(")]
            sValue = u"[[ setLang" + sObjName + self.aListFields[itemSelectedPos].replace("/",".") + ")" " ]]"
            oCurObj.Items = (sKey, sValue)
            oCurObj.update()
            self.endExecute()
        elif itemSelected != "" and txtUName != "" :
            oInputList = doc.createInstance("com.sun.star.text.TextField.DropDown")
            sObjName = self.getComboBoxText("cmbVariable")
            sObjName = sObjName[:sObjName.find("(")]

            widget = ( cursor.TextTable and cursor.TextTable.getCellByName( cursor.Cell.CellName ) or doc.Text )

            sValue = u"[[setLang" + "(" + sObjName + self.aListFields[itemSelectedPos].replace("/",".") +")" " ]]"
            oInputList.Items = (sKey, sValue)
            widget.insertTextContent(cursor, oInputList, False)

            self.endExecute()
        else:
            ErrorDialog("Please Fill appropriate data in Name field \nor select particular value from the list of fields")

    def btnCancel_clicked( self, oActionEvent ):
        self.endExecute()


class AddAttachment(unohelper.Base, XJobExecutor ):
    Kind = {
        'PDF' : 'pdf',
        'OpenOffice': 'sxw',
    }
    @login_required
    def __init__(self, ctx):
        self.ctx     = ctx
        self.module  = MODULE_NAME
        self.version = MODULE_VERSION

        self.aSearchResult = []
        desktop = getDesktop()
        oDoc2 = desktop.getCurrentComponent()
        docinfo = oDoc2.getDocumentInfo()

        if docinfo.getUserFieldValue(2) != "" and docinfo.getUserFieldValue(3) != "":
            self.win = DBModalDialog(60, 50, 180, 70, "Add Attachment to Server")
            self.win.addFixedText("lblResourceType", 2 , 5, 100, 10, "Select Appropriate Resource Type:")
            self.win.addComboListBox("lstResourceType", -2, 25, 176, 15, True)
            self.win.addButton('btnOkWithoutInformation', -2 , -5, 25 , 15, 'OK',
                               actionListenerProc = self.btnOkWithoutInformation_clicked )
        else:
            self.win = DBModalDialog(60, 50, 180, 190, "Add Attachment to Server")
            self.win.addFixedText("lblModuleName", 2 , 9, 42, 20, "Select Module:")
            self.win.addComboListBox("lstmodel", -2, 5, 134, 15, True)
            self.lstModel = self.win.getControl( "lstmodel" )
            self.dModel = {}

            # Open a new connexion to the server
            if not Object('ir.module.module').search([('name', '=', 'base_report_designer'),
                                                      ('state', '=', 'installed')]):
                # If the module 'base_report_model' is not installed, use the default model
                self.dModel = {
                    "Partner":'res.partner',
                }
            else:
                # FIXME The base report model object does not exist in the system WTF !
                res = Object('base.report.model').select([], ['name', 'model_id'])
                models = Object('ir.model').read([ x['model_id'][0] for x in res ], ['model'])
                models = dict((model['id'], model['model']) for model in models )
                self.dModel = dict( (x['name'], models[x['model_id'][0]]) for x in res)

            for item in self.dModel.keys():
                self.lstModel.addItem(item, self.lstModel.getItemCount())

            self.win.addFixedText("lblSearchName", 2 , 25, 60, 10, "Enter Search String:")
            self.win.addEdit("txtSearchName", 2, 35, 149, 15,)
            self.win.addButton('btnSearch', -2 , 35, 25 , 15, 'Search',
                               actionListenerProc = self.btnSearch_clicked )

            self.win.addFixedText("lblSearchRecord", 2 , 55, 60, 10, "Search Result:")
            self.win.addComboListBox("lstResource", -2, 65, 176, 70, False )
            self.lstResource = self.win.getControl( "lstResource" )

            self.win.addFixedText("lblResourceType", 2 , 137, 100, 20, "Select Appropriate Resource Type:")
            self.win.addComboListBox("lstResourceType", -2, 147, 176, 15, True )

            self.win.addButton('btnOkWithInformation', -2 , -5, 25 , 15, 'OK',
                               actionListenerProc = self.btnOkWithInformation_clicked )

        self.lstResourceType = self.win.getControl( "lstResourceType" )
        for kind in self.Kind.keys():
            self.lstResourceType.addItem( kind, self.lstResourceType.getItemCount() )

        self.win.addButton('btnCancel', -2 - 27 , -5 , 30 , 15, 'Cancel',
                           actionListenerProc = self.btnCancel_clicked )
        self.win.doModalDialog("lstResourceType", self.Kind.keys()[0])

    def btnSearch_clicked( self, oActionEvent ):
        modelSelectedItem = self.win.getListBoxSelectedItem("lstmodel")
        if modelSelectedItem == "":
            return

        self.aSearchResult = Object(self.dModel[modelSelectedItem]).name_search(self.win.getEditText("txtSearchName"))
        self.win.removeListBoxItems("lstResource", 0, self.win.getListBoxItemCount("lstResource"))
        if self.aSearchResult == []:
            ErrorDialog("No search result found !!!", "", "Search ERROR" )
            return

        for result in self.aSearchResult:
            self.lstResource.addItem(result[1], result[0])

    def _send_attachment( self, name, data, res_model, res_id ):
        params = {
            'name': name,
            'datas': base64.encodestring( data ),
            'datas_fname': name,
            'res_model' : res_model,
            'res_id' : int(res_id),
        }

        return Object('ir.attachment').create(params)

    def send_attachment( self, model, resource_id ):
        desktop = getDesktop()
        oDoc2 = desktop.getCurrentComponent()

        if oDoc2.getURL() == "":
            ErrorDialog("Please save your file", "", "Saving ERROR" )
            return None

        url = oDoc2.getURL()
        if self.Kind[self.win.getListBoxSelectedItem("lstResourceType")] == "pdf":
            url = self.doc2pdf(url[7:])

        if url is None:
            ErrorDialog( "Ploblem in creating PDF", "", "PDF Error" )
            return None

        url = url[7:]
        data = read_data_from_file( get_absolute_file_path( url ) )
        return self._send_attachment( os.path.basename( url ), data, model, resource_id )

    def btnOkWithoutInformation_clicked( self, oActionEvent ):
        desktop = getDesktop()
        oDoc2 = desktop.getCurrentComponent()
        docinfo = oDoc2.getDocumentInfo()

        if self.win.getListBoxSelectedItem("lstResourceType") == "":
            ErrorDialog("Please select resource type", "", "Selection ERROR" )
            return

        self.send_attachment( docinfo.getUserFieldValue(3), docinfo.getUserFieldValue(2) )
        self.win.endExecute()

    def btnOkWithInformation_clicked(self, oActionEvent):
        if self.win.getListBoxSelectedItem("lstResourceType") == "":
            ErrorDialog( "Please select resource type", "", "Selection ERROR" )
            return

        if self.win.getListBoxSelectedItem("lstResource") == "" or self.win.getListBoxSelectedItem("lstmodel") == "":
            ErrorDialog("Please select Model and Resource","","Selection ERROR")
            return

        resourceid = None
        for s in self.aSearchResult:
            if s[1] == self.win.getListBoxSelectedItem("lstResource"):
                resourceid = s[0]
                break

        if resourceid is None:
            ErrorDialog("No resource selected !!!", "", "Resource ERROR" )
            return

        self.send_attachment( self.dModel[self.win.getListBoxSelectedItem('lstmodel')], resourceid )
        self.win.endExecute()

    def btnCancel_clicked( self, oActionEvent ):
        self.win.endExecute()

    def doc2pdf(self, strFile):
        strUrl = convertToURL( strFile )
        desktop = getDesktop()
        oDoc = desktop.loadComponentFromURL(strUrl, "_blank", 0, Array(makePropertyValue("Hidden", True)))
        if oDoc:
            strFilterSubName = ""
            # select appropriate filter
            if oDoc.supportsService("com.sun.star.presentation.PresentationDocument"):
                strFilterSubName = "impress_pdf_Export"
            elif oDoc.supportsService("com.sun.star.sheet.SpreadsheetDocument"):
                strFilterSubName = "calc_pdf_Export"
            elif oDoc.supportsService("com.sun.star.text.WebDocument"):
                strFilterSubName = "writer_web_pdf_Export"
            elif oDoc.supportsService("com.sun.star.text.GlobalDocument"):
                strFilterSubName = "writer_globaldocument_pdf_Export"
            elif oDoc.supportsService("com.sun.star.text.TextDocument"):
                strFilterSubName = "writer_pdf_Export"
            elif oDoc.supportsService("com.sun.star.drawing.DrawingDocument"):
                strFilterSubName = "draw_pdf_Export"
            elif oDoc.supportsService("com.sun.star.formula.FormulaProperties"):
                strFilterSubName = "math_pdf_Export"
            elif oDoc.supportsService("com.sun.star.chart.ChartDocument"):
                strFilterSubName = "chart_pdf_Export"

            filename = len(strFilterSubName) > 0 and convertToURL( os.path.splitext( strFile )[0] + ".pdf" ) or None

            if len(strFilterSubName) > 0:
                oDoc.storeToURL(filename,
                                Array(makePropertyValue("FilterName", strFilterSubName ),
                                      makePropertyValue("CompressMode", "1" ))
                               )

            oDoc.close(True)
            # Can be None if len(strFilterSubName) <= 0
            return filename

if __name__ == "__main__":
    os.system( "ooffice '-accept=socket,host=localhost,port=2002;urp;'" )
    RepeatIn()
    Fields()
    Expression()
    modify(None)
    ServerParameter(None)
    NewReport(None)
    ModifyExistingReport(None)
    SendtoServer(None)
    About(None)
    ConvertBracesToField(None)
    ConvertFieldsToBraces(None)
    ExportToRML(None)
    AddLang()
    AddAttachment(None)
else:
    g_ImplementationHelper = unohelper.ImplementationHelper()
    g_ImplementationHelper.addImplementation( RepeatIn, "org.openoffice.openerp.report.repeatln", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( Fields, "org.openoffice.openerp.report.fields", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( Expression, "org.openoffice.openerp.report.expression", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( modify, "org.openoffice.openerp.report.modify", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( NewReport, "org.openoffice.openerp.report.opennewreport", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( ModifyExistingReport, "org.openoffice.openerp.report.modifyreport", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( ServerParameter, "org.openoffice.openerp.report.serverparam", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( SendtoServer, "org.openoffice.openerp.report.sendtoserver", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( About, "org.openoffice.openerp.report.about", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( ConvertBracesToField, "org.openoffice.openerp.report.convertBF", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( ConvertFieldsToBraces, "org.openoffice.openerp.report.convertFB", ("com.sun.star.task.Job",),) 
    g_ImplementationHelper.addImplementation( ExportToRML, "org.openoffice.openerp.report.exporttorml", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( AddLang, "org.openoffice.openerp.report.langtag", ("com.sun.star.task.Job",),)
    g_ImplementationHelper.addImplementation( AddAttachment, "org.openoffice.openerp.report.addattachment", ("com.sun.star.task.Job",),)
